// $ANTLR 2.7.7 (2006-11-01): "grammar.g" -> "ProtocolParser.java"$

/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */
	package de.bsvrz.pua.prot.interpreter;

	import java.io.*;
	import java.awt.event.*;
	import antlr.*;
	import antlr.debug.misc.*;

	import java.util.ArrayList;
	import java.util.HashSet;
	import java.util.List;
	import de.bsvrz.pua.prot.util.ExpressionTree;
	import de.bsvrz.pua.prot.util.ExpressionLeaf;
	import de.bsvrz.pua.prot.exceptions.CriticalParserException;
	import de.bsvrz.pua.prot.exceptions.SemanticErrorException;
	import de.bsvrz.pua.prot.util.ProcessingInformation;
	import de.bsvrz.pua.prot.util.ProcessingParameter;
	import de.bsvrz.sys.funclib.losb.datk.ProtocolType;
	import de.bsvrz.sys.funclib.losb.datk.NoChangeMarker;
	import de.bsvrz.pua.prot.util.attributes.DurationAttributeDescription;

	import de.bsvrz.pua.prot.interpreter.semantics.Semantics;
	import de.bsvrz.pua.prot.interpreter.semantics.AggregationSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.AttributeGroupSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.AttributeSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.ExpressionSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.MiscellaneousSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.RealAttributeSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.ScopeSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.TempAttributeSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.SymbolTable;

	import de.bsvrz.dav.daf.main.config.DataModel;


import antlr.TokenBuffer;
import antlr.TokenStreamException;
import antlr.TokenStreamIOException;
import antlr.ANTLRException;
import antlr.LLkParser;
import antlr.Token;
import antlr.TokenStream;
import antlr.RecognitionException;
import antlr.NoViableAltException;
import antlr.MismatchedTokenException;
import antlr.SemanticException;
import antlr.ParserSharedInputState;
import antlr.collections.impl.BitSet;
import antlr.collections.AST;
import java.util.Hashtable;
import antlr.ASTFactory;
import antlr.ASTPair;
import antlr.collections.impl.ASTArray;

/**
 * @author Kappich + Kniss Systemberatung Verkehr und Technik
 * @author O. Weiss
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $$Revision: 1.9 $$ / $$Date: 2007/12/17 15:22:59 $$ / ($$Author: martinh $$)
 */
public class ProtocolParser extends antlr.LLkParser       implements ProtocolTokensTokenTypes
 {


	/**
	 * Jeder Bereich darf nur einmal im Skript vorkommen, jedoch in beliebiger Reihenfolge
	 * [0] = spalten, [1] = filter, [2] = objekte, [3] = definitionen
	 */
	private boolean _definition[] = {false, false, false, false};

	/**
	 * Einige Schlüsselwörter im Filter-Bereich dürfen nur einmal verwendet werden
	 * [0] = Einschränkung, [1] = Anwendung, [2] = Nachfilter
	 */
	private boolean _filter[] = {false, false, false};

	/**
	 *  Symboltabelle
	 */
	private SymbolTable _symbolTable = null;

	/**
	 *  Objekt zur Semantiküberprüfung
	 */
	private Semantics _semantics = null;

	private AggregationSemantics _aggregationSemantics;
	private AttributeGroupSemantics _attributeGroupSemantics;
	private AttributeSemantics _attributeSemantics;
	private ExpressionSemantics _expressionSemantics;
	private MiscellaneousSemantics _miscSemantics;
	private RealAttributeSemantics _realAttributeSemantics;
	private ScopeSemantics _scopeSemantics;
	private TempAttributeSemantics _tempAttributeSemantics;

	/**
	 * @return Objekt zur Semantiküberprüfung
	 */
	public Semantics getSemantics()
	{
		return _semantics;
	}

	/**
	 * Diese Methode muss zur Initialisierung des Parsers aufgerufen werden
	 * @param model		Datenmodell
 	 * @param parameter	Informationen zum Auswerten des Skripts
	 * @param pi		Objekt, in dem die gewonnenen Informationen abgelegt werden
	 */
	public void init(DataModel model, ProcessingParameter parameter, ProcessingInformation pi) throws CriticalParserException
	{
		_symbolTable = new SymbolTable(model, parameter);

		this.setASTNodeClass("de.bsvrz.pua.prot.interpreter.AdvancedAST");

		_semantics = new Semantics(model, _symbolTable, pi);
		_aggregationSemantics = _semantics.getAggregationSemantics();
		_attributeGroupSemantics = _semantics.getAttributeGroupSemantics();
		_attributeSemantics = _semantics.getAttributeSemantics();
		_expressionSemantics = _semantics.getExpressionSemantics();
		_miscSemantics = _semantics.getMiscSemantics();
		_realAttributeSemantics = _semantics.getRealAttributeSemantics();
		_scopeSemantics = _semantics.getScopeSemantics();
		_tempAttributeSemantics = _semantics.getTempAttributeSemantics();

		_symbolTable.setCheck(_semantics.getCheck());
	}

protected ProtocolParser(TokenBuffer tokenBuf, int k) {
  super(tokenBuf,k);
  tokenNames = _tokenNames;
  buildTokenTypeASTClassMap();
  astFactory = new ASTFactory(getTokenTypeToASTClassMap());
}

public ProtocolParser(TokenBuffer tokenBuf) {
  this(tokenBuf,2);
}

protected ProtocolParser(TokenStream lexer, int k) {
  super(lexer,k);
  tokenNames = _tokenNames;
  buildTokenTypeASTClassMap();
  astFactory = new ASTFactory(getTokenTypeToASTClassMap());
}

public ProtocolParser(TokenStream lexer) {
  this(lexer,2);
}

public ProtocolParser(ParserSharedInputState state) {
  super(state,2);
  tokenNames = _tokenNames;
  buildTokenTypeASTClassMap();
  astFactory = new ASTFactory(getTokenTypeToASTClassMap());
}

	public final void skript() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST skript_AST = null;
		
				Token token;
			
		
		{
		_loop3:
		do {
			switch ( LA(1)) {
			case Spalten:
			{
				token=spaltenDefinition();
				astFactory.addASTChild(currentAST, returnAST);
				_miscSemantics.check(token,_definition, 0);
				break;
			}
			case Filter:
			{
				token=filterDefinition();
				astFactory.addASTChild(currentAST, returnAST);
				_miscSemantics.check(token, _definition, 1);
				break;
			}
			case Standards:
			{
				token=standardsListe();
				astFactory.addASTChild(currentAST, returnAST);
				_miscSemantics.check(token, _definition, 2);
				break;
			}
			case Definitionen:
			{
				token=definitionenListe();
				astFactory.addASTChild(currentAST, returnAST);
				_miscSemantics.check(token, _definition, 3);
				break;
			}
			default:
			{
				break _loop3;
			}
			}
		} while (true);
		}
		match(Token.EOF_TYPE);
		skript_AST = (AdvancedAST)currentAST.root;
		returnAST = skript_AST;
	}
	
	public final Token  spaltenDefinition() throws RecognitionException, TokenStreamException {
		Token token;
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST spaltenDefinition_AST = null;
		Token  a = null;
		AdvancedAST a_AST = null;
		
		a = LT(1);
		a_AST = (AdvancedAST)astFactory.create(a);
		astFactory.makeASTRoot(currentAST, a_AST);
		match(Spalten);
		match(Doppelpunkt);
		token = a;
		spaltenMitBlock();
		astFactory.addASTChild(currentAST, returnAST);
		spaltenDefinition_AST = (AdvancedAST)currentAST.root;
		returnAST = spaltenDefinition_AST;
		return token;
	}
	
	public final Token  filterDefinition() throws RecognitionException, TokenStreamException {
		Token token;
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST filterDefinition_AST = null;
		Token  a = null;
		AdvancedAST a_AST = null;
		
		a = LT(1);
		a_AST = (AdvancedAST)astFactory.create(a);
		astFactory.makeASTRoot(currentAST, a_AST);
		match(Filter);
		match(Doppelpunkt);
		token = a;
		filterMitBlock();
		astFactory.addASTChild(currentAST, returnAST);
		filterDefinition_AST = (AdvancedAST)currentAST.root;
		returnAST = filterDefinition_AST;
		return token;
	}
	
	public final Token  standardsListe() throws RecognitionException, TokenStreamException {
		Token token;
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST standardsListe_AST = null;
		Token  a = null;
		AdvancedAST a_AST = null;
		
		a = LT(1);
		a_AST = (AdvancedAST)astFactory.create(a);
		astFactory.makeASTRoot(currentAST, a_AST);
		match(Standards);
		match(Doppelpunkt);
		token = a;
		standardsMitBlock();
		astFactory.addASTChild(currentAST, returnAST);
		standardsListe_AST = (AdvancedAST)currentAST.root;
		returnAST = standardsListe_AST;
		return token;
	}
	
	public final Token  definitionenListe() throws RecognitionException, TokenStreamException {
		Token token;
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST definitionenListe_AST = null;
		Token  a = null;
		AdvancedAST a_AST = null;
		
		a = LT(1);
		a_AST = (AdvancedAST)astFactory.create(a);
		astFactory.makeASTRoot(currentAST, a_AST);
		match(Definitionen);
		match(Doppelpunkt);
		token = a;
		definitionenMitBlock();
		astFactory.addASTChild(currentAST, returnAST);
		definitionenListe_AST = (AdvancedAST)currentAST.root;
		returnAST = definitionenListe_AST;
		return token;
	}
	
	public final void spaltenMitBlock() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST spaltenMitBlock_AST = null;
		
		{
		_loop7:
		do {
			switch ( LA(1)) {
			case Mit:
			{
				mitKonstruktSpalte();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case Attribut:
			{
				attributSpalte();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case TempAttribut:
			{
				tempAttributSpalte();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case Attributgruppe:
			{
				attributgruppenSpalte();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			default:
			{
				break _loop7;
			}
			}
		} while (true);
		}
		spaltenMitBlock_AST = (AdvancedAST)currentAST.root;
		returnAST = spaltenMitBlock_AST;
	}
	
	public final void mitKonstruktSpalte() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST mitKonstruktSpalte_AST = null;
		
		AdvancedAST tmp110_AST = null;
		tmp110_AST = (AdvancedAST)astFactory.create(LT(1));
		astFactory.makeASTRoot(currentAST, tmp110_AST);
		match(Mit);
		qualifizierung();
		astFactory.addASTChild(currentAST, returnAST);
		spaltenMitBlock();
		astFactory.addASTChild(currentAST, returnAST);
		match(Ende);
		match(Mit);
		mitKonstruktSpalte_AST = (AdvancedAST)currentAST.root;
		returnAST = mitKonstruktSpalte_AST;
	}
	
	public final void attributSpalte() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST attributSpalte_AST = null;
		
			String alsName = null;
		
		
		if ((LA(1)==Attribut) && (_tokenSet_0.member(LA(2)))) {
			AdvancedAST tmp113_AST = null;
			tmp113_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp113_AST);
			match(Attribut);
			{
			switch ( LA(1)) {
			case NichtAuffuellenA:
			case NichtAuffuellenB:
			{
				nichtAuffuellen();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case Name:
			case NameMitKlammern:
			case Fragezeichen:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			attribut();
			astFactory.addASTChild(currentAST, returnAST);
			{
			switch ( LA(1)) {
			case Als:
			{
				alsDefinition();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case EOF:
			case Spalten:
			case Mit:
			case Ende:
			case Attribut:
			case TempAttribut:
			case Attributgruppe:
			case Filter:
			case Standards:
			case Definitionen:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			attributSpalte_AST = (AdvancedAST)currentAST.root;
		}
		else if ((LA(1)==Attribut) && (LA(2)==Zeitdauer)) {
			match(Attribut);
			AdvancedAST tmp115_AST = null;
			tmp115_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp115_AST);
			match(Zeitdauer);
			{
			switch ( LA(1)) {
			case Als:
			{
				alsName=alsDefinition();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case EOF:
			case Spalten:
			case Mit:
			case Ende:
			case Attribut:
			case TempAttribut:
			case Attributgruppe:
			case Filter:
			case Standards:
			case Definitionen:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			_symbolTable.addDurationAttribute(alsName);
			attributSpalte_AST = (AdvancedAST)currentAST.root;
		}
		else {
			throw new NoViableAltException(LT(1), getFilename());
		}
		
		returnAST = attributSpalte_AST;
	}
	
	public final void tempAttributSpalte() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST tempAttributSpalte_AST = null;
		Token  n = null;
		AdvancedAST n_AST = null;
		
				String alsName = null;
			
		
		AdvancedAST tmp116_AST = null;
		tmp116_AST = (AdvancedAST)astFactory.create(LT(1));
		astFactory.makeASTRoot(currentAST, tmp116_AST);
		match(TempAttribut);
		n = LT(1);
		n_AST = (AdvancedAST)astFactory.create(n);
		astFactory.addASTChild(currentAST, n_AST);
		match(Name);
		{
		switch ( LA(1)) {
		case Als:
		{
			alsName=alsDefinition();
			astFactory.addASTChild(currentAST, returnAST);
			break;
		}
		case EOF:
		case Spalten:
		case Mit:
		case Ende:
		case Attribut:
		case TempAttribut:
		case Attributgruppe:
		case Filter:
		case Standards:
		case Definitionen:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		_symbolTable.addTempAttribute(n.getText(), alsName);
		tempAttributSpalte_AST = (AdvancedAST)currentAST.root;
		returnAST = tempAttributSpalte_AST;
	}
	
	public final void attributgruppenSpalte() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST attributgruppenSpalte_AST = null;
		
		AdvancedAST tmp117_AST = null;
		tmp117_AST = (AdvancedAST)astFactory.create(LT(1));
		astFactory.makeASTRoot(currentAST, tmp117_AST);
		match(Attributgruppe);
		{
		switch ( LA(1)) {
		case NichtAuffuellenA:
		case NichtAuffuellenB:
		{
			nichtAuffuellen();
			astFactory.addASTChild(currentAST, returnAST);
			break;
		}
		case Name:
		case NameMitKlammern:
		case Fragezeichen:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		{
		_loop28:
		do {
			if ((LA(1)==Name) && (LA(2)==Doppelpunkt)) {
				AdvancedAST tmp118_AST = null;
				tmp118_AST = (AdvancedAST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp118_AST);
				match(Name);
				match(Doppelpunkt);
			}
			else {
				break _loop28;
			}
			
		} while (true);
		}
		aspektName();
		astFactory.addASTChild(currentAST, returnAST);
		{
		switch ( LA(1)) {
		case Als:
		{
			alsDefinition();
			astFactory.addASTChild(currentAST, returnAST);
			break;
		}
		case EOF:
		case Spalten:
		case Mit:
		case Ende:
		case Attribut:
		case TempAttribut:
		case Attributgruppe:
		case Filter:
		case Standards:
		case Definitionen:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		attributgruppenSpalte_AST = (AdvancedAST)currentAST.root;
		returnAST = attributgruppenSpalte_AST;
	}
	
	public final void qualifizierung() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST qualifizierung_AST = null;
		
		AdvancedAST tmp120_AST = null;
		tmp120_AST = (AdvancedAST)astFactory.create(LT(1));
		astFactory.makeASTRoot(currentAST, tmp120_AST);
		match(Name);
		{
		switch ( LA(1)) {
		case Doppelpunkt:
		{
			match(Doppelpunkt);
			AdvancedAST tmp122_AST = null;
			tmp122_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp122_AST);
			match(Name);
			{
			switch ( LA(1)) {
			case Doppelpunkt:
			{
				match(Doppelpunkt);
				aspektName();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case Mit:
			case Ende:
			case Name:
			case NameMitKlammern:
			case Fragezeichen:
			case Attribut:
			case Zeitdauer:
			case TempAttribut:
			case Attributgruppe:
			case Einschraenkung:
			case EinschraenkungUml:
			case Anwendung:
			case Nachfilter:
			case Protokollzeitraum:
			case Protokollart:
			case UnveraendertKennungA:
			case UnveraendertKennungB:
			case Pseudoobjekt:
			case Aggregationsfunktion:
			case Alias:
			case NichtAuffuellenA:
			case NichtAuffuellenB:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			break;
		}
		case Mit:
		case Ende:
		case Name:
		case NameMitKlammern:
		case Fragezeichen:
		case Attribut:
		case Zeitdauer:
		case TempAttribut:
		case Attributgruppe:
		case Einschraenkung:
		case EinschraenkungUml:
		case Anwendung:
		case Nachfilter:
		case Protokollzeitraum:
		case Protokollart:
		case UnveraendertKennungA:
		case UnveraendertKennungB:
		case Pseudoobjekt:
		case Aggregationsfunktion:
		case Alias:
		case NichtAuffuellenA:
		case NichtAuffuellenB:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		qualifizierung_AST = (AdvancedAST)currentAST.root;
		returnAST = qualifizierung_AST;
	}
	
	public final void aspektName() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST aspektName_AST = null;
		
		switch ( LA(1)) {
		case Name:
		case NameMitKlammern:
		{
			{
			switch ( LA(1)) {
			case Name:
			{
				AdvancedAST tmp124_AST = null;
				tmp124_AST = (AdvancedAST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp124_AST);
				match(Name);
				break;
			}
			case NameMitKlammern:
			{
				AdvancedAST tmp125_AST = null;
				tmp125_AST = (AdvancedAST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp125_AST);
				match(NameMitKlammern);
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			aspektName_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Fragezeichen:
		{
			AdvancedAST tmp126_AST = null;
			tmp126_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp126_AST);
			match(Fragezeichen);
			AdvancedAST tmp127_AST = null;
			tmp127_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp127_AST);
			match(GanzZahl);
			aspektName_AST = (AdvancedAST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = aspektName_AST;
	}
	
	public final void nichtAuffuellen() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST nichtAuffuellen_AST = null;
		
		switch ( LA(1)) {
		case NichtAuffuellenA:
		{
			AdvancedAST tmp128_AST = null;
			tmp128_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp128_AST);
			match(NichtAuffuellenA);
			nichtAuffuellen_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case NichtAuffuellenB:
		{
			AdvancedAST tmp129_AST = null;
			tmp129_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp129_AST);
			match(NichtAuffuellenB);
			nichtAuffuellen_AST = (AdvancedAST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = nichtAuffuellen_AST;
	}
	
	public final void attribut() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST attribut_AST = null;
		
		aspektName();
		astFactory.addASTChild(currentAST, returnAST);
		{
		_loop21:
		do {
			if ((LA(1)==Doppelpunkt)) {
				AdvancedAST tmp130_AST = null;
				tmp130_AST = (AdvancedAST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp130_AST);
				match(Doppelpunkt);
				{
				aspektName();
				astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop21;
			}
			
		} while (true);
		}
		attribut_AST = (AdvancedAST)currentAST.root;
		returnAST = attribut_AST;
	}
	
	public final String  alsDefinition() throws RecognitionException, TokenStreamException {
		String alsName = null;
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST alsDefinition_AST = null;
		Token  z = null;
		AdvancedAST z_AST = null;
		
		AdvancedAST tmp131_AST = null;
		tmp131_AST = (AdvancedAST)astFactory.create(LT(1));
		astFactory.makeASTRoot(currentAST, tmp131_AST);
		match(Als);
		z = LT(1);
		z_AST = (AdvancedAST)astFactory.create(z);
		astFactory.addASTChild(currentAST, z_AST);
		match(Zeichenkette);
		alsName = z.getText().substring(1,z.getText().length()-1);
		alsDefinition_AST = (AdvancedAST)currentAST.root;
		returnAST = alsDefinition_AST;
		return alsName;
	}
	
	public final void filterMitBlock() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST filterMitBlock_AST = null;
		
				Token token;
			
		
		{
		_loop33:
		do {
			switch ( LA(1)) {
			case Mit:
			{
				mitKonstruktFilter();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case Name:
			case NameMitKlammern:
			case Fragezeichen:
			case Zeitdauer:
			case NichtAuffuellenA:
			case NichtAuffuellenB:
			{
				aggregationsBlock();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case Einschraenkung:
			case EinschraenkungUml:
			{
				token=einschraenkungsAnweisung();
				astFactory.addASTChild(currentAST, returnAST);
				_miscSemantics.check(token, _filter,0);
				break;
			}
			case Anwendung:
			{
				token=anwendungsAnweisung();
				astFactory.addASTChild(currentAST, returnAST);
				_miscSemantics.check(token, _filter,1);
				break;
			}
			case Nachfilter:
			{
				token=nachfilterAnweisung();
				astFactory.addASTChild(currentAST, returnAST);
				_miscSemantics.check(token, _filter,2);
				break;
			}
			default:
			{
				break _loop33;
			}
			}
		} while (true);
		}
		filterMitBlock_AST = (AdvancedAST)currentAST.root;
		returnAST = filterMitBlock_AST;
	}
	
	public final void mitKonstruktFilter() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST mitKonstruktFilter_AST = null;
		
		AdvancedAST tmp132_AST = null;
		tmp132_AST = (AdvancedAST)astFactory.create(LT(1));
		astFactory.makeASTRoot(currentAST, tmp132_AST);
		match(Mit);
		qualifizierung();
		astFactory.addASTChild(currentAST, returnAST);
		filterMitBlock();
		astFactory.addASTChild(currentAST, returnAST);
		match(Ende);
		match(Mit);
		mitKonstruktFilter_AST = (AdvancedAST)currentAST.root;
		returnAST = mitKonstruktFilter_AST;
	}
	
	public final void aggregationsBlock() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST aggregationsBlock_AST = null;
		
		switch ( LA(1)) {
		case Name:
		case NameMitKlammern:
		case Fragezeichen:
		case NichtAuffuellenA:
		case NichtAuffuellenB:
		{
			{
			switch ( LA(1)) {
			case NichtAuffuellenA:
			case NichtAuffuellenB:
			{
				nichtAuffuellen();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case Name:
			case NameMitKlammern:
			case Fragezeichen:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			aspektName();
			astFactory.addASTChild(currentAST, returnAST);
			{
			_loop38:
			do {
				if ((LA(1)==Doppelpunkt) && ((LA(2) >= Name && LA(2) <= Fragezeichen))) {
					match(Doppelpunkt);
					aspektName();
					astFactory.addASTChild(currentAST, returnAST);
				}
				else {
					break _loop38;
				}
				
			} while (true);
			}
			AdvancedAST tmp136_AST = null;
			tmp136_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp136_AST);
			match(Doppelpunkt);
			AdvancedAST tmp137_AST = null;
			tmp137_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp137_AST);
			match(Aggregation);
			{
			AdvancedAST tmp138_AST = null;
			tmp138_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp138_AST);
			match(Name);
			}
			{
			_loop42:
			do {
				if ((LA(1)==Komma)) {
					AdvancedAST tmp139_AST = null;
					tmp139_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.addASTChild(currentAST, tmp139_AST);
					match(Komma);
					{
					AdvancedAST tmp140_AST = null;
					tmp140_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.addASTChild(currentAST, tmp140_AST);
					match(Name);
					}
				}
				else {
					break _loop42;
				}
				
			} while (true);
			}
			aggregationsBlock_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Zeitdauer:
		{
			AdvancedAST tmp141_AST = null;
			tmp141_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp141_AST);
			match(Zeitdauer);
			match(Doppelpunkt);
			match(Aggregation);
			{
			AdvancedAST tmp144_AST = null;
			tmp144_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp144_AST);
			match(Name);
			}
			{
			_loop46:
			do {
				if ((LA(1)==Komma)) {
					AdvancedAST tmp145_AST = null;
					tmp145_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.addASTChild(currentAST, tmp145_AST);
					match(Komma);
					{
					AdvancedAST tmp146_AST = null;
					tmp146_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.addASTChild(currentAST, tmp146_AST);
					match(Name);
					}
				}
				else {
					break _loop46;
				}
				
			} while (true);
			}
			aggregationsBlock_AST = (AdvancedAST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = aggregationsBlock_AST;
	}
	
	public final Token  einschraenkungsAnweisung() throws RecognitionException, TokenStreamException {
		Token token=null;
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST einschraenkungsAnweisung_AST = null;
		Token  a1 = null;
		AdvancedAST a1_AST = null;
		Token  a2 = null;
		AdvancedAST a2_AST = null;
		
		{
		switch ( LA(1)) {
		case Einschraenkung:
		{
			a1 = LT(1);
			a1_AST = (AdvancedAST)astFactory.create(a1);
			astFactory.makeASTRoot(currentAST, a1_AST);
			match(Einschraenkung);
			token = a1;
			break;
		}
		case EinschraenkungUml:
		{
			a2 = LT(1);
			a2_AST = (AdvancedAST)astFactory.create(a2);
			astFactory.makeASTRoot(currentAST, a2_AST);
			match(EinschraenkungUml);
			token = a2;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		{
		switch ( LA(1)) {
		case Verschmelzen:
		{
			AdvancedAST tmp147_AST = null;
			tmp147_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp147_AST);
			match(Verschmelzen);
			break;
		}
		case Name:
		case NameMitKlammern:
		case Fragezeichen:
		case GanzZahl:
		case Zeitdauer:
		case Zeichenkette:
		case KlammerAuf:
		case Minus:
		case Nicht:
		case KommaZahl:
		case EckigeKlammerAuf:
		case Potenz:
		case Exp:
		case Ln:
		case Min:
		case Max:
		case Abs:
		case IstFehler:
		case IstZahl:
		case IstZustand:
		case IstLeer:
		case IstText:
		case IstWahrheitswert:
		case Falsch:
		case Wahr:
		case NichtAuffuellenA:
		case NichtAuffuellenB:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		ausdruck();
		astFactory.addASTChild(currentAST, returnAST);
		einschraenkungsAnweisung_AST = (AdvancedAST)currentAST.root;
		returnAST = einschraenkungsAnweisung_AST;
		return token;
	}
	
	public final Token  anwendungsAnweisung() throws RecognitionException, TokenStreamException {
		Token token;
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST anwendungsAnweisung_AST = null;
		Token  a = null;
		AdvancedAST a_AST = null;
		
		a = LT(1);
		a_AST = (AdvancedAST)astFactory.create(a);
		astFactory.makeASTRoot(currentAST, a_AST);
		match(Anwendung);
		anwendungsArt();
		astFactory.addASTChild(currentAST, returnAST);
		{
		_loop52:
		do {
			if ((LA(1)==Komma)) {
				match(Komma);
				anwendungsArt();
				astFactory.addASTChild(currentAST, returnAST);
			}
			else {
				break _loop52;
			}
			
		} while (true);
		}
		token = a;
		anwendungsAnweisung_AST = (AdvancedAST)currentAST.root;
		returnAST = anwendungsAnweisung_AST;
		return token;
	}
	
	public final Token  nachfilterAnweisung() throws RecognitionException, TokenStreamException {
		Token token;
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST nachfilterAnweisung_AST = null;
		Token  a = null;
		AdvancedAST a_AST = null;
		
		a = LT(1);
		a_AST = (AdvancedAST)astFactory.create(a);
		astFactory.makeASTRoot(currentAST, a_AST);
		match(Nachfilter);
		AdvancedAST tmp149_AST = null;
		tmp149_AST = (AdvancedAST)astFactory.create(LT(1));
		astFactory.addASTChild(currentAST, tmp149_AST);
		match(Zeichenkette);
		token = a;
		nachfilterAnweisung_AST = (AdvancedAST)currentAST.root;
		returnAST = nachfilterAnweisung_AST;
		return token;
	}
	
	public final void ausdruck() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST ausdruck_AST = null;
		
		oderAusdruck();
		astFactory.addASTChild(currentAST, returnAST);
		ausdruck_AST = (AdvancedAST)currentAST.root;
		returnAST = ausdruck_AST;
	}
	
	public final void anwendungsArt() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST anwendungsArt_AST = null;
		
		switch ( LA(1)) {
		case Gesamt:
		{
			AdvancedAST tmp150_AST = null;
			tmp150_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp150_AST);
			match(Gesamt);
			{
			switch ( LA(1)) {
			case KlammerAuf:
			{
				match(KlammerAuf);
				{
				switch ( LA(1)) {
				case Objekt:
				{
					AdvancedAST tmp152_AST = null;
					tmp152_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.addASTChild(currentAST, tmp152_AST);
					match(Objekt);
					break;
				}
				case Spalte:
				{
					AdvancedAST tmp153_AST = null;
					tmp153_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.addASTChild(currentAST, tmp153_AST);
					match(Spalte);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				match(KlammerZu);
				break;
			}
			case EOF:
			case Spalten:
			case Mit:
			case Ende:
			case Name:
			case NameMitKlammern:
			case Fragezeichen:
			case Zeitdauer:
			case Filter:
			case Komma:
			case Einschraenkung:
			case EinschraenkungUml:
			case Anwendung:
			case Nachfilter:
			case Standards:
			case Definitionen:
			case NichtAuffuellenA:
			case NichtAuffuellenB:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			anwendungsArt_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Intervall:
		{
			AdvancedAST tmp155_AST = null;
			tmp155_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp155_AST);
			match(Intervall);
			{
			switch ( LA(1)) {
			case KlammerAuf:
			{
				match(KlammerAuf);
				{
				switch ( LA(1)) {
				case Objekt:
				{
					AdvancedAST tmp157_AST = null;
					tmp157_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.addASTChild(currentAST, tmp157_AST);
					match(Objekt);
					break;
				}
				case Spalte:
				{
					AdvancedAST tmp158_AST = null;
					tmp158_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.addASTChild(currentAST, tmp158_AST);
					match(Spalte);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				match(KlammerZu);
				break;
			}
			case EOF:
			case Spalten:
			case Mit:
			case Ende:
			case Name:
			case NameMitKlammern:
			case Fragezeichen:
			case Zeitdauer:
			case Filter:
			case Komma:
			case Einschraenkung:
			case EinschraenkungUml:
			case Anwendung:
			case Nachfilter:
			case Standards:
			case Definitionen:
			case NichtAuffuellenA:
			case NichtAuffuellenB:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			anwendungsArt_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Liste:
		{
			AdvancedAST tmp160_AST = null;
			tmp160_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp160_AST);
			match(Liste);
			anwendungsArt_AST = (AdvancedAST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = anwendungsArt_AST;
	}
	
	public final void standardsMitBlock() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST standardsMitBlock_AST = null;
		
		{
		_loop62:
		do {
			switch ( LA(1)) {
			case Mit:
			{
				mitKonstruktObjekte();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case Name:
			{
				objektDefinition();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case Pseudoobjekt:
			{
				pseudoObjektDefinition();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case Protokollzeitraum:
			{
				protokollzeitraumDefinition();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case Protokollart:
			{
				protokollartDefinition();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case Fragezeichen:
			{
				standardbindungDefinition();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case UnveraendertKennungA:
			case UnveraendertKennungB:
			{
				unveraendertKennungDefinition();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			default:
			{
				break _loop62;
			}
			}
		} while (true);
		}
		standardsMitBlock_AST = (AdvancedAST)currentAST.root;
		returnAST = standardsMitBlock_AST;
	}
	
	public final void mitKonstruktObjekte() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST mitKonstruktObjekte_AST = null;
		
		AdvancedAST tmp161_AST = null;
		tmp161_AST = (AdvancedAST)astFactory.create(LT(1));
		astFactory.makeASTRoot(currentAST, tmp161_AST);
		match(Mit);
		qualifizierung();
		astFactory.addASTChild(currentAST, returnAST);
		standardsMitBlock();
		astFactory.addASTChild(currentAST, returnAST);
		match(Ende);
		match(Mit);
		mitKonstruktObjekte_AST = (AdvancedAST)currentAST.root;
		returnAST = mitKonstruktObjekte_AST;
	}
	
	public final void objektDefinition() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST objektDefinition_AST = null;
		Token  n = null;
		AdvancedAST n_AST = null;
		Token  m = null;
		AdvancedAST m_AST = null;
		
				ArrayList list = new ArrayList();
			
		
		n = LT(1);
		n_AST = (AdvancedAST)astFactory.create(n);
		astFactory.makeASTRoot(currentAST, n_AST);
		match(Name);
		list.add(n.getText());
		{
		_loop66:
		do {
			if ((LA(1)==Doppelpunkt)) {
				match(Doppelpunkt);
				m = LT(1);
				m_AST = (AdvancedAST)astFactory.create(m);
				astFactory.addASTChild(currentAST, m_AST);
				match(Name);
				list.add(m.getText());
			}
			else {
				break _loop66;
			}
			
		} while (true);
		}
		_miscSemantics.addStandardObjects(n,list);
		objektDefinition_AST = (AdvancedAST)currentAST.root;
		returnAST = objektDefinition_AST;
	}
	
	public final void pseudoObjektDefinition() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST pseudoObjektDefinition_AST = null;
		Token  t = null;
		AdvancedAST t_AST = null;
		Token  TY1 = null;
		AdvancedAST TY1_AST = null;
		Token  TY2 = null;
		AdvancedAST TY2_AST = null;
		Token  O1 = null;
		AdvancedAST O1_AST = null;
		Token  O2 = null;
		AdvancedAST O2_AST = null;
		
		ArrayList types = new ArrayList();
				 ArrayList objects = new ArrayList();
		t = LT(1);
		t_AST = (AdvancedAST)astFactory.create(t);
		astFactory.makeASTRoot(currentAST, t_AST);
		match(Pseudoobjekt);
		match(KlammerAuf);
		TY1 = LT(1);
		TY1_AST = (AdvancedAST)astFactory.create(TY1);
		astFactory.addASTChild(currentAST, TY1_AST);
		match(Name);
		types.add(TY1.getText());
		{
		_loop76:
		do {
			if ((LA(1)==Komma)) {
				match(Komma);
				TY2 = LT(1);
				TY2_AST = (AdvancedAST)astFactory.create(TY2);
				astFactory.addASTChild(currentAST, TY2_AST);
				match(Name);
				types.add(TY2.getText());
			}
			else {
				break _loop76;
			}
			
		} while (true);
		}
		match(KlammerZu);
		AdvancedAST tmp168_AST = null;
		tmp168_AST = (AdvancedAST)astFactory.create(LT(1));
		astFactory.addASTChild(currentAST, tmp168_AST);
		match(Gleich);
		{
		int _cnt82=0;
		_loop82:
		do {
			if ((LA(1)==KlammerAuf)) {
				match(KlammerAuf);
				{
				switch ( LA(1)) {
				case Name:
				{
					O1 = LT(1);
					O1_AST = (AdvancedAST)astFactory.create(O1);
					astFactory.addASTChild(currentAST, O1_AST);
					match(Name);
					break;
				}
				case Komma:
				case KlammerZu:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				objects.add((O1 == null) ? "" : O1.getText()); O1 = null;
				{
				_loop81:
				do {
					if ((LA(1)==Komma)) {
						match(Komma);
						{
						switch ( LA(1)) {
						case Name:
						{
							O2 = LT(1);
							O2_AST = (AdvancedAST)astFactory.create(O2);
							astFactory.addASTChild(currentAST, O2_AST);
							match(Name);
							break;
						}
						case Komma:
						case KlammerZu:
						{
							break;
						}
						default:
						{
							throw new NoViableAltException(LT(1), getFilename());
						}
						}
						}
						objects.add((O2 == null) ? "" : O2.getText()); O2 = null;
					}
					else {
						break _loop81;
					}
					
				} while (true);
				}
				match(KlammerZu);
			}
			else {
				if ( _cnt82>=1 ) { break _loop82; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt82++;
		} while (true);
		}
		_symbolTable.addPseudoObjectDefinition(t, types, objects );
		pseudoObjektDefinition_AST = (AdvancedAST)currentAST.root;
		returnAST = pseudoObjektDefinition_AST;
	}
	
	public final void protokollzeitraumDefinition() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST protokollzeitraumDefinition_AST = null;
		Token  ds = null;
		AdvancedAST ds_AST = null;
		Token  zs = null;
		AdvancedAST zs_AST = null;
		Token  de = null;
		AdvancedAST de_AST = null;
		Token  ze = null;
		AdvancedAST ze_AST = null;
		
		AdvancedAST tmp172_AST = null;
		tmp172_AST = (AdvancedAST)astFactory.create(LT(1));
		astFactory.makeASTRoot(currentAST, tmp172_AST);
		match(Protokollzeitraum);
		ds = LT(1);
		ds_AST = (AdvancedAST)astFactory.create(ds);
		astFactory.addASTChild(currentAST, ds_AST);
		match(Datum);
		match(Komma);
		zs = LT(1);
		zs_AST = (AdvancedAST)astFactory.create(zs);
		astFactory.addASTChild(currentAST, zs_AST);
		match(Zeit);
		match(Minus);
		de = LT(1);
		de_AST = (AdvancedAST)astFactory.create(de);
		astFactory.addASTChild(currentAST, de_AST);
		match(Datum);
		match(Komma);
		ze = LT(1);
		ze_AST = (AdvancedAST)astFactory.create(ze);
		astFactory.addASTChild(currentAST, ze_AST);
		match(Zeit);
		_symbolTable.addStandardPeriod(ds.getText(),zs.getText(),de.getText(),ze.getText());
		protokollzeitraumDefinition_AST = (AdvancedAST)currentAST.root;
		returnAST = protokollzeitraumDefinition_AST;
	}
	
	public final void protokollartDefinition() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST protokollartDefinition_AST = null;
		
		AdvancedAST tmp176_AST = null;
		tmp176_AST = (AdvancedAST)astFactory.create(LT(1));
		astFactory.makeASTRoot(currentAST, tmp176_AST);
		match(Protokollart);
		{
		switch ( LA(1)) {
		case Aenderungsprotokoll:
		{
			AdvancedAST tmp177_AST = null;
			tmp177_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp177_AST);
			match(Aenderungsprotokoll);
			break;
		}
		case AenderungsprotokollUml:
		{
			AdvancedAST tmp178_AST = null;
			tmp178_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp178_AST);
			match(AenderungsprotokollUml);
			break;
		}
		case Zustandsprotokoll:
		{
			AdvancedAST tmp179_AST = null;
			tmp179_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp179_AST);
			match(Zustandsprotokoll);
			break;
		}
		case Ereignisprotokoll:
		{
			AdvancedAST tmp180_AST = null;
			tmp180_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp180_AST);
			match(Ereignisprotokoll);
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		protokollartDefinition_AST = (AdvancedAST)currentAST.root;
		returnAST = protokollartDefinition_AST;
	}
	
	public final void standardbindungDefinition() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST standardbindungDefinition_AST = null;
		Token  f = null;
		AdvancedAST f_AST = null;
		Token  z = null;
		AdvancedAST z_AST = null;
		Token  n = null;
		AdvancedAST n_AST = null;
		
		f = LT(1);
		f_AST = (AdvancedAST)astFactory.create(f);
		astFactory.makeASTRoot(currentAST, f_AST);
		match(Fragezeichen);
		z = LT(1);
		z_AST = (AdvancedAST)astFactory.create(z);
		astFactory.addASTChild(currentAST, z_AST);
		match(GanzZahl);
		match(Gleich);
		n = LT(1);
		n_AST = (AdvancedAST)astFactory.create(n);
		astFactory.addASTChild(currentAST, n_AST);
		match(Name);
		_symbolTable.addStandardBinding(f.getText() + z.getText(), n.getText());
		standardbindungDefinition_AST = (AdvancedAST)currentAST.root;
		returnAST = standardbindungDefinition_AST;
	}
	
	public final void unveraendertKennungDefinition() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST unveraendertKennungDefinition_AST = null;
		
		switch ( LA(1)) {
		case UnveraendertKennungA:
		{
			AdvancedAST tmp182_AST = null;
			tmp182_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp182_AST);
			match(UnveraendertKennungA);
			AdvancedAST tmp183_AST = null;
			tmp183_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp183_AST);
			match(Pro);
			{
			switch ( LA(1)) {
			case Zeile:
			{
				AdvancedAST tmp184_AST = null;
				tmp184_AST = (AdvancedAST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp184_AST);
				match(Zeile);
				break;
			}
			case Zelle:
			{
				AdvancedAST tmp185_AST = null;
				tmp185_AST = (AdvancedAST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp185_AST);
				match(Zelle);
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			unveraendertKennungDefinition_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case UnveraendertKennungB:
		{
			AdvancedAST tmp186_AST = null;
			tmp186_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp186_AST);
			match(UnveraendertKennungB);
			AdvancedAST tmp187_AST = null;
			tmp187_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp187_AST);
			match(Pro);
			{
			switch ( LA(1)) {
			case Zeile:
			{
				AdvancedAST tmp188_AST = null;
				tmp188_AST = (AdvancedAST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp188_AST);
				match(Zeile);
				break;
			}
			case Zelle:
			{
				AdvancedAST tmp189_AST = null;
				tmp189_AST = (AdvancedAST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp189_AST);
				match(Zelle);
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			unveraendertKennungDefinition_AST = (AdvancedAST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = unveraendertKennungDefinition_AST;
	}
	
	public final void definitionenMitBlock() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST definitionenMitBlock_AST = null;
		
		{
		_loop86:
		do {
			switch ( LA(1)) {
			case Mit:
			{
				mitKonstruktDefinitionen();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case TempAttribut:
			{
				tempAttributDefinition();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case Aggregationsfunktion:
			{
				aggregatsfunktionDefinition();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case Alias:
			{
				aliasDefinition();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			default:
			{
				break _loop86;
			}
			}
		} while (true);
		}
		definitionenMitBlock_AST = (AdvancedAST)currentAST.root;
		returnAST = definitionenMitBlock_AST;
	}
	
	public final void mitKonstruktDefinitionen() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST mitKonstruktDefinitionen_AST = null;
		
		AdvancedAST tmp190_AST = null;
		tmp190_AST = (AdvancedAST)astFactory.create(LT(1));
		astFactory.makeASTRoot(currentAST, tmp190_AST);
		match(Mit);
		qualifizierung();
		astFactory.addASTChild(currentAST, returnAST);
		definitionenMitBlock();
		astFactory.addASTChild(currentAST, returnAST);
		match(Ende);
		match(Mit);
		mitKonstruktDefinitionen_AST = (AdvancedAST)currentAST.root;
		returnAST = mitKonstruktDefinitionen_AST;
	}
	
	public final void tempAttributDefinition() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST tempAttributDefinition_AST = null;
		Token  O = null;
		AdvancedAST O_AST = null;
		Token  N = null;
		AdvancedAST N_AST = null;
		
		AdvancedAST tmp193_AST = null;
		tmp193_AST = (AdvancedAST)astFactory.create(LT(1));
		astFactory.makeASTRoot(currentAST, tmp193_AST);
		match(TempAttribut);
		O = LT(1);
		O_AST = (AdvancedAST)astFactory.create(O);
		astFactory.addASTChild(currentAST, O_AST);
		match(Name);
		match(Doppelpunkt);
		N = LT(1);
		N_AST = (AdvancedAST)astFactory.create(N);
		astFactory.addASTChild(currentAST, N_AST);
		match(Name);
		match(Gleich);
		ausdruck();
		astFactory.addASTChild(currentAST, returnAST);
		tempAttributDefinition_AST = (AdvancedAST)currentAST.root;
		returnAST = tempAttributDefinition_AST;
	}
	
	public final void aggregatsfunktionDefinition() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST aggregatsfunktionDefinition_AST = null;
		Token  a = null;
		AdvancedAST a_AST = null;
		Token  n = null;
		AdvancedAST n_AST = null;
		Token  z = null;
		AdvancedAST z_AST = null;
		
		a = LT(1);
		a_AST = (AdvancedAST)astFactory.create(a);
		astFactory.makeASTRoot(currentAST, a_AST);
		match(Aggregationsfunktion);
		n = LT(1);
		n_AST = (AdvancedAST)astFactory.create(n);
		astFactory.addASTChild(currentAST, n_AST);
		match(Name);
		match(Gleich);
		z = LT(1);
		z_AST = (AdvancedAST)astFactory.create(z);
		astFactory.addASTChild(currentAST, z_AST);
		match(Zeichenkette);
		_symbolTable.addAggregationDefinition(a, n.getText(), z.getText().substring(1,z.getText().length()-1));
		aggregatsfunktionDefinition_AST = (AdvancedAST)currentAST.root;
		returnAST = aggregatsfunktionDefinition_AST;
	}
	
	public final void aliasDefinition() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST aliasDefinition_AST = null;
		Token  N = null;
		AdvancedAST N_AST = null;
		Token  M = null;
		AdvancedAST M_AST = null;
		
		AdvancedAST tmp197_AST = null;
		tmp197_AST = (AdvancedAST)astFactory.create(LT(1));
		astFactory.makeASTRoot(currentAST, tmp197_AST);
		match(Alias);
		N = LT(1);
		N_AST = (AdvancedAST)astFactory.create(N);
		astFactory.addASTChild(currentAST, N_AST);
		match(Name);
		match(Gleich);
		M = LT(1);
		M_AST = (AdvancedAST)astFactory.create(M);
		astFactory.addASTChild(currentAST, M_AST);
		match(Name);
		_symbolTable.addAliasDefinition(M, N.getText(),M.getText());
		aliasDefinition_AST = (AdvancedAST)currentAST.root;
		returnAST = aliasDefinition_AST;
	}
	
	public final void oderAusdruck() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST oderAusdruck_AST = null;
		
		undAusdruck();
		astFactory.addASTChild(currentAST, returnAST);
		{
		_loop94:
		do {
			if ((LA(1)==Oder)) {
				AdvancedAST tmp199_AST = null;
				tmp199_AST = (AdvancedAST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp199_AST);
				match(Oder);
				undAusdruck();
				astFactory.addASTChild(currentAST, returnAST);
			}
			else {
				break _loop94;
			}
			
		} while (true);
		}
		oderAusdruck_AST = (AdvancedAST)currentAST.root;
		returnAST = oderAusdruck_AST;
	}
	
	public final void undAusdruck() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST undAusdruck_AST = null;
		
		gleichheitsAusdruck();
		astFactory.addASTChild(currentAST, returnAST);
		{
		_loop97:
		do {
			if ((LA(1)==Und)) {
				AdvancedAST tmp200_AST = null;
				tmp200_AST = (AdvancedAST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp200_AST);
				match(Und);
				gleichheitsAusdruck();
				astFactory.addASTChild(currentAST, returnAST);
			}
			else {
				break _loop97;
			}
			
		} while (true);
		}
		undAusdruck_AST = (AdvancedAST)currentAST.root;
		returnAST = undAusdruck_AST;
	}
	
	public final void gleichheitsAusdruck() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST gleichheitsAusdruck_AST = null;
		
		vergleichsAusdruck();
		astFactory.addASTChild(currentAST, returnAST);
		{
		_loop101:
		do {
			if ((LA(1)==Gleich||LA(1)==Ungleich)) {
				{
				switch ( LA(1)) {
				case Gleich:
				{
					AdvancedAST tmp201_AST = null;
					tmp201_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp201_AST);
					match(Gleich);
					break;
				}
				case Ungleich:
				{
					AdvancedAST tmp202_AST = null;
					tmp202_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp202_AST);
					match(Ungleich);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				vergleichsAusdruck();
				astFactory.addASTChild(currentAST, returnAST);
			}
			else {
				break _loop101;
			}
			
		} while (true);
		}
		gleichheitsAusdruck_AST = (AdvancedAST)currentAST.root;
		returnAST = gleichheitsAusdruck_AST;
	}
	
	public final void vergleichsAusdruck() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST vergleichsAusdruck_AST = null;
		
		additionsAusdruck();
		astFactory.addASTChild(currentAST, returnAST);
		{
		_loop105:
		do {
			if (((LA(1) >= Kleiner && LA(1) <= GroesserGleich))) {
				{
				switch ( LA(1)) {
				case Kleiner:
				{
					AdvancedAST tmp203_AST = null;
					tmp203_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp203_AST);
					match(Kleiner);
					break;
				}
				case Groesser:
				{
					AdvancedAST tmp204_AST = null;
					tmp204_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp204_AST);
					match(Groesser);
					break;
				}
				case KleinerGleich:
				{
					AdvancedAST tmp205_AST = null;
					tmp205_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp205_AST);
					match(KleinerGleich);
					break;
				}
				case GroesserGleich:
				{
					AdvancedAST tmp206_AST = null;
					tmp206_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp206_AST);
					match(GroesserGleich);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				additionsAusdruck();
				astFactory.addASTChild(currentAST, returnAST);
			}
			else {
				break _loop105;
			}
			
		} while (true);
		}
		vergleichsAusdruck_AST = (AdvancedAST)currentAST.root;
		returnAST = vergleichsAusdruck_AST;
	}
	
	public final void additionsAusdruck() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST additionsAusdruck_AST = null;
		
		multiplikationsAusdruck();
		astFactory.addASTChild(currentAST, returnAST);
		{
		_loop109:
		do {
			if ((LA(1)==Minus||LA(1)==Plus||LA(1)==Ampersand)) {
				{
				switch ( LA(1)) {
				case Plus:
				{
					AdvancedAST tmp207_AST = null;
					tmp207_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp207_AST);
					match(Plus);
					break;
				}
				case Minus:
				{
					AdvancedAST tmp208_AST = null;
					tmp208_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp208_AST);
					match(Minus);
					break;
				}
				case Ampersand:
				{
					AdvancedAST tmp209_AST = null;
					tmp209_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp209_AST);
					match(Ampersand);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				multiplikationsAusdruck();
				astFactory.addASTChild(currentAST, returnAST);
			}
			else {
				break _loop109;
			}
			
		} while (true);
		}
		additionsAusdruck_AST = (AdvancedAST)currentAST.root;
		returnAST = additionsAusdruck_AST;
	}
	
	public final void multiplikationsAusdruck() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST multiplikationsAusdruck_AST = null;
		
		negationsAusdruck();
		astFactory.addASTChild(currentAST, returnAST);
		{
		_loop113:
		do {
			if (((LA(1) >= Mal && LA(1) <= Modulo))) {
				{
				switch ( LA(1)) {
				case Mal:
				{
					AdvancedAST tmp210_AST = null;
					tmp210_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp210_AST);
					match(Mal);
					break;
				}
				case Dividiert:
				{
					AdvancedAST tmp211_AST = null;
					tmp211_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp211_AST);
					match(Dividiert);
					break;
				}
				case Div:
				{
					AdvancedAST tmp212_AST = null;
					tmp212_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp212_AST);
					match(Div);
					break;
				}
				case Modulo:
				{
					AdvancedAST tmp213_AST = null;
					tmp213_AST = (AdvancedAST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp213_AST);
					match(Modulo);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				negationsAusdruck();
				astFactory.addASTChild(currentAST, returnAST);
			}
			else {
				break _loop113;
			}
			
		} while (true);
		}
		multiplikationsAusdruck_AST = (AdvancedAST)currentAST.root;
		returnAST = multiplikationsAusdruck_AST;
	}
	
	public final void negationsAusdruck() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST negationsAusdruck_AST = null;
		
		switch ( LA(1)) {
		case Nicht:
		{
			AdvancedAST tmp214_AST = null;
			tmp214_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp214_AST);
			match(Nicht);
			negationsAusdruck();
			astFactory.addASTChild(currentAST, returnAST);
			negationsAusdruck_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Minus:
		{
			AdvancedAST tmp215_AST = null;
			tmp215_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp215_AST);
			match(Minus);
			negationsAusdruck();
			astFactory.addASTChild(currentAST, returnAST);
			negationsAusdruck_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Name:
		case NameMitKlammern:
		case Fragezeichen:
		case GanzZahl:
		case Zeitdauer:
		case Zeichenkette:
		case KlammerAuf:
		case KommaZahl:
		case EckigeKlammerAuf:
		case Potenz:
		case Exp:
		case Ln:
		case Min:
		case Max:
		case Abs:
		case IstFehler:
		case IstZahl:
		case IstZustand:
		case IstLeer:
		case IstText:
		case IstWahrheitswert:
		case Falsch:
		case Wahr:
		case NichtAuffuellenA:
		case NichtAuffuellenB:
		{
			erweiterterAusdruck();
			astFactory.addASTChild(currentAST, returnAST);
			negationsAusdruck_AST = (AdvancedAST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = negationsAusdruck_AST;
	}
	
	public final void erweiterterAusdruck() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST erweiterterAusdruck_AST = null;
		
		switch ( LA(1)) {
		case Name:
		case NameMitKlammern:
		case Fragezeichen:
		case GanzZahl:
		case Zeitdauer:
		case Zeichenkette:
		case KommaZahl:
		case Falsch:
		case Wahr:
		case NichtAuffuellenA:
		case NichtAuffuellenB:
		{
			basisAusdruck();
			astFactory.addASTChild(currentAST, returnAST);
			erweiterterAusdruck_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case KlammerAuf:
		case EckigeKlammerAuf:
		case Potenz:
		case Exp:
		case Ln:
		case Min:
		case Max:
		case Abs:
		case IstFehler:
		case IstZahl:
		case IstZustand:
		case IstLeer:
		case IstText:
		case IstWahrheitswert:
		{
			erweiterbarerBasisAusdruck();
			astFactory.addASTChild(currentAST, returnAST);
			erweiterterAusdruck_AST = (AdvancedAST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = erweiterterAusdruck_AST;
	}
	
	public final void basisAusdruck() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST basisAusdruck_AST = null;
		
		switch ( LA(1)) {
		case GanzZahl:
		{
			AdvancedAST tmp216_AST = null;
			tmp216_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp216_AST);
			match(GanzZahl);
			basisAusdruck_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case KommaZahl:
		{
			AdvancedAST tmp217_AST = null;
			tmp217_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp217_AST);
			match(KommaZahl);
			basisAusdruck_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Falsch:
		case Wahr:
		{
			wahrheitswert();
			astFactory.addASTChild(currentAST, returnAST);
			basisAusdruck_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Zeichenkette:
		{
			AdvancedAST tmp218_AST = null;
			tmp218_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp218_AST);
			match(Zeichenkette);
			basisAusdruck_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Zeitdauer:
		{
			AdvancedAST tmp219_AST = null;
			tmp219_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp219_AST);
			match(Zeitdauer);
			basisAusdruck_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Name:
		case NameMitKlammern:
		case Fragezeichen:
		case NichtAuffuellenA:
		case NichtAuffuellenB:
		{
			{
			switch ( LA(1)) {
			case NichtAuffuellenA:
			case NichtAuffuellenB:
			{
				nichtAuffuellen();
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case Name:
			case NameMitKlammern:
			case Fragezeichen:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			attribut();
			astFactory.addASTChild(currentAST, returnAST);
			basisAusdruck_AST = (AdvancedAST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = basisAusdruck_AST;
	}
	
	public final void erweiterbarerBasisAusdruck() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST erweiterbarerBasisAusdruck_AST = null;
		
		switch ( LA(1)) {
		case Potenz:
		case Exp:
		case Ln:
		case Min:
		case Max:
		case Abs:
		case IstFehler:
		case IstZahl:
		case IstZustand:
		case IstLeer:
		case IstText:
		case IstWahrheitswert:
		{
			funktionsAufruf();
			astFactory.addASTChild(currentAST, returnAST);
			erweiterbarerBasisAusdruck_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case KlammerAuf:
		{
			match(KlammerAuf);
			ausdruck();
			astFactory.addASTChild(currentAST, returnAST);
			match(KlammerZu);
			erweiterbarerBasisAusdruck_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case EckigeKlammerAuf:
		{
			AdvancedAST tmp222_AST = null;
			tmp222_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp222_AST);
			match(EckigeKlammerAuf);
			{
			switch ( LA(1)) {
			case Name:
			case NameMitKlammern:
			case Fragezeichen:
			case GanzZahl:
			case Zeitdauer:
			case Zeichenkette:
			case KlammerAuf:
			case Minus:
			case Nicht:
			case KommaZahl:
			case EckigeKlammerAuf:
			case Potenz:
			case Exp:
			case Ln:
			case Min:
			case Max:
			case Abs:
			case IstFehler:
			case IstZahl:
			case IstZustand:
			case IstLeer:
			case IstText:
			case IstWahrheitswert:
			case Falsch:
			case Wahr:
			case NichtAuffuellenA:
			case NichtAuffuellenB:
			{
				ausdruck();
				astFactory.addASTChild(currentAST, returnAST);
				{
				_loop121:
				do {
					if ((LA(1)==Komma)) {
						match(Komma);
						ausdruck();
						astFactory.addASTChild(currentAST, returnAST);
					}
					else {
						break _loop121;
					}
					
				} while (true);
				}
				break;
			}
			case EckigeKlammerZu:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(EckigeKlammerZu);
			erweiterbarerBasisAusdruck_AST = (AdvancedAST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = erweiterbarerBasisAusdruck_AST;
	}
	
	public final void wahrheitswert() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST wahrheitswert_AST = null;
		
		switch ( LA(1)) {
		case Falsch:
		{
			AdvancedAST tmp225_AST = null;
			tmp225_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp225_AST);
			match(Falsch);
			wahrheitswert_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Wahr:
		{
			AdvancedAST tmp226_AST = null;
			tmp226_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp226_AST);
			match(Wahr);
			wahrheitswert_AST = (AdvancedAST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = wahrheitswert_AST;
	}
	
	public final void funktionsAufruf() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST funktionsAufruf_AST = null;
		
		switch ( LA(1)) {
		case Potenz:
		{
			AdvancedAST tmp227_AST = null;
			tmp227_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp227_AST);
			match(Potenz);
			match(KlammerAuf);
			ausdruck();
			astFactory.addASTChild(currentAST, returnAST);
			match(Komma);
			ausdruck();
			astFactory.addASTChild(currentAST, returnAST);
			match(KlammerZu);
			funktionsAufruf_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Exp:
		{
			AdvancedAST tmp231_AST = null;
			tmp231_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp231_AST);
			match(Exp);
			match(KlammerAuf);
			ausdruck();
			astFactory.addASTChild(currentAST, returnAST);
			match(KlammerZu);
			funktionsAufruf_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Ln:
		{
			AdvancedAST tmp234_AST = null;
			tmp234_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp234_AST);
			match(Ln);
			match(KlammerAuf);
			ausdruck();
			astFactory.addASTChild(currentAST, returnAST);
			match(KlammerZu);
			funktionsAufruf_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Min:
		{
			AdvancedAST tmp237_AST = null;
			tmp237_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp237_AST);
			match(Min);
			match(KlammerAuf);
			ausdruck();
			astFactory.addASTChild(currentAST, returnAST);
			{
			int _cnt124=0;
			_loop124:
			do {
				if ((LA(1)==Komma)) {
					match(Komma);
					ausdruck();
					astFactory.addASTChild(currentAST, returnAST);
				}
				else {
					if ( _cnt124>=1 ) { break _loop124; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				
				_cnt124++;
			} while (true);
			}
			match(KlammerZu);
			funktionsAufruf_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Max:
		{
			AdvancedAST tmp241_AST = null;
			tmp241_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp241_AST);
			match(Max);
			match(KlammerAuf);
			ausdruck();
			astFactory.addASTChild(currentAST, returnAST);
			{
			int _cnt126=0;
			_loop126:
			do {
				if ((LA(1)==Komma)) {
					match(Komma);
					ausdruck();
					astFactory.addASTChild(currentAST, returnAST);
				}
				else {
					if ( _cnt126>=1 ) { break _loop126; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				
				_cnt126++;
			} while (true);
			}
			match(KlammerZu);
			funktionsAufruf_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case Abs:
		{
			AdvancedAST tmp245_AST = null;
			tmp245_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp245_AST);
			match(Abs);
			match(KlammerAuf);
			ausdruck();
			astFactory.addASTChild(currentAST, returnAST);
			match(KlammerZu);
			funktionsAufruf_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case IstFehler:
		{
			AdvancedAST tmp248_AST = null;
			tmp248_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp248_AST);
			match(IstFehler);
			match(KlammerAuf);
			ausdruck();
			astFactory.addASTChild(currentAST, returnAST);
			match(KlammerZu);
			funktionsAufruf_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case IstZahl:
		{
			AdvancedAST tmp251_AST = null;
			tmp251_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp251_AST);
			match(IstZahl);
			match(KlammerAuf);
			ausdruck();
			astFactory.addASTChild(currentAST, returnAST);
			match(KlammerZu);
			funktionsAufruf_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case IstZustand:
		{
			AdvancedAST tmp254_AST = null;
			tmp254_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp254_AST);
			match(IstZustand);
			match(KlammerAuf);
			ausdruck();
			astFactory.addASTChild(currentAST, returnAST);
			match(KlammerZu);
			funktionsAufruf_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case IstLeer:
		{
			AdvancedAST tmp257_AST = null;
			tmp257_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp257_AST);
			match(IstLeer);
			match(KlammerAuf);
			ausdruck();
			astFactory.addASTChild(currentAST, returnAST);
			match(KlammerZu);
			funktionsAufruf_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case IstText:
		{
			AdvancedAST tmp260_AST = null;
			tmp260_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp260_AST);
			match(IstText);
			match(KlammerAuf);
			ausdruck();
			astFactory.addASTChild(currentAST, returnAST);
			match(KlammerZu);
			funktionsAufruf_AST = (AdvancedAST)currentAST.root;
			break;
		}
		case IstWahrheitswert:
		{
			AdvancedAST tmp263_AST = null;
			tmp263_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp263_AST);
			match(IstWahrheitswert);
			match(KlammerAuf);
			ausdruck();
			astFactory.addASTChild(currentAST, returnAST);
			match(KlammerZu);
			funktionsAufruf_AST = (AdvancedAST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = funktionsAufruf_AST;
	}
	
	public final void unveraendertKennung() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AdvancedAST unveraendertKennung_AST = null;
		
		{
		switch ( LA(1)) {
		case UnveraendertKennungA:
		{
			AdvancedAST tmp266_AST = null;
			tmp266_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp266_AST);
			match(UnveraendertKennungA);
			break;
		}
		case UnveraendertKennungB:
		{
			AdvancedAST tmp267_AST = null;
			tmp267_AST = (AdvancedAST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp267_AST);
			match(UnveraendertKennungB);
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		unveraendertKennung_AST = (AdvancedAST)currentAST.root;
		returnAST = unveraendertKennung_AST;
	}
	
	
	public static final String[] _tokenNames = {
		"<0>",
		"EOF",
		"<2>",
		"NULL_TREE_LOOKAHEAD",
		"Spalten",
		"Doppelpunkt",
		"Mit",
		"Ende",
		"Name",
		"NameMitKlammern",
		"Fragezeichen",
		"GanzZahl",
		"Attribut",
		"Zeitdauer",
		"Als",
		"Zeichenkette",
		"TempAttribut",
		"Attributgruppe",
		"Filter",
		"Aggregation",
		"Komma",
		"Einschraenkung",
		"EinschraenkungUml",
		"Verschmelzen",
		"Anwendung",
		"Gesamt",
		"KlammerAuf",
		"Objekt",
		"Spalte",
		"KlammerZu",
		"Intervall",
		"Liste",
		"Nachfilter",
		"Standards",
		"Protokollzeitraum",
		"Datum",
		"Zeit",
		"Minus",
		"Protokollart",
		"Aenderungsprotokoll",
		"AenderungsprotokollUml",
		"Zustandsprotokoll",
		"Ereignisprotokoll",
		"Gleich",
		"UnveraendertKennungA",
		"Pro",
		"Zeile",
		"Zelle",
		"UnveraendertKennungB",
		"Pseudoobjekt",
		"Definitionen",
		"Aggregationsfunktion",
		"Alias",
		"Oder",
		"Und",
		"Ungleich",
		"Kleiner",
		"Groesser",
		"KleinerGleich",
		"GroesserGleich",
		"Plus",
		"Ampersand",
		"Mal",
		"Dividiert",
		"Div",
		"Modulo",
		"Nicht",
		"KommaZahl",
		"EckigeKlammerAuf",
		"EckigeKlammerZu",
		"Potenz",
		"Exp",
		"Ln",
		"Min",
		"Max",
		"Abs",
		"IstFehler",
		"IstZahl",
		"IstZustand",
		"IstLeer",
		"IstText",
		"IstWahrheitswert",
		"Falsch",
		"Wahr",
		"NichtAuffuellenA",
		"NichtAuffuellenB"
	};
	
	protected void buildTokenTypeASTClassMap() {
		tokenTypeToASTClassMap=null;
	};
	
	private static final long[] mk_tokenSet_0() {
		long[] data = { 1792L, 3145728L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_0 = new BitSet(mk_tokenSet_0());
	
	}
