header {
/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Contact Information:
 * beck et al. projects GmbH
 * Theresienhoehe 13
 * 80336 Munich
 * Phone: +49-89-5442530
 * mailto: info@bea.de
 */
	package de.bsvrz.pua.prot.interpreter;

	import java.io.*;
	import java.awt.event.*;
	import antlr.*;
	import antlr.debug.misc.*;

	import java.util.ArrayList;
	import java.util.HashSet;
	import java.util.List;
	import de.bsvrz.pua.prot.util.ExpressionTree;
	import de.bsvrz.pua.prot.util.ExpressionLeaf;
	import de.bsvrz.pua.prot.exceptions.CriticalParserException;
	import de.bsvrz.pua.prot.exceptions.SemanticErrorException;
	import de.bsvrz.pua.prot.util.ProcessingInformation;
	import de.bsvrz.pua.prot.util.ProcessingParameter;
	import de.bsvrz.sys.funclib.losb.datk.ProtocolType;
	import de.bsvrz.sys.funclib.losb.datk.NoChangeMarker;
	import de.bsvrz.pua.prot.util.attributes.DurationAttributeDescription;

	import de.bsvrz.pua.prot.interpreter.semantics.Semantics;
	import de.bsvrz.pua.prot.interpreter.semantics.AggregationSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.AttributeGroupSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.AttributeSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.ExpressionSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.MiscellaneousSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.RealAttributeSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.ScopeSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.TempAttributeSemantics;
	import de.bsvrz.pua.prot.interpreter.semantics.SymbolTable;

	import de.bsvrz.dav.daf.main.config.DataModel;

}
/**
 * @author Kappich + Kniss Systemberatung Verkehr und Technik
 * @author O. Weiss
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $$Revision: 1.9 $$ / $$Date: 2007/12/17 15:22:59 $$ / ($$Author: martinh $$)
 */
class ProtocolParser extends Parser;
options {
	k =2;				// Lookahead: Two Token. Do not change to one, otherwise the 'attribut' rule will generate errors. (even if antlr doesn't complain when comiled)
	exportVocab=ProtocolTokens;
	buildAST = true;
	defaultErrorHandler=false;	//Andernfalls werden vom Parser keine Exceptions geworfen, und der Skript-Interpreter merkt nicht, dass es zu einem Fehler kam.
	ASTLabelType = "AdvancedAST";
}

//Set the correct node type
//tokens {

//	Mit<AST=de.bsvrz.pua.prot.interpreter.AdvancedAST>;
//}

{

	/**
	 * Jeder Bereich darf nur einmal im Skript vorkommen, jedoch in beliebiger Reihenfolge
	 * [0] = spalten, [1] = filter, [2] = objekte, [3] = definitionen
	 */
	private boolean _definition[] = {false, false, false, false};

	/**
	 * Einige Schlüsselwörter im Filter-Bereich dürfen nur einmal verwendet werden
	 * [0] = Einschränkung, [1] = Anwendung, [2] = Nachfilter
	 */
	private boolean _filter[] = {false, false, false};

	/**
	 *  Symboltabelle
	 */
	private SymbolTable _symbolTable = null;

	/**
	 *  Objekt zur Semantiküberprüfung
	 */
	private Semantics _semantics = null;

	private AggregationSemantics _aggregationSemantics;
	private AttributeGroupSemantics _attributeGroupSemantics;
	private AttributeSemantics _attributeSemantics;
	private ExpressionSemantics _expressionSemantics;
	private MiscellaneousSemantics _miscSemantics;
	private RealAttributeSemantics _realAttributeSemantics;
	private ScopeSemantics _scopeSemantics;
	private TempAttributeSemantics _tempAttributeSemantics;

	/**
	 * @return Objekt zur Semantiküberprüfung
	 */
	public Semantics getSemantics()
	{
		return _semantics;
	}

	/**
	 * Diese Methode muss zur Initialisierung des Parsers aufgerufen werden
	 * @param model		Datenmodell
 	 * @param parameter	Informationen zum Auswerten des Skripts
	 * @param pi		Objekt, in dem die gewonnenen Informationen abgelegt werden
	 */
	public void init(DataModel model, ProcessingParameter parameter, ProcessingInformation pi) throws CriticalParserException
	{
		_symbolTable = new SymbolTable(model, parameter);

		this.setASTNodeClass("de.bsvrz.pua.prot.interpreter.AdvancedAST");

		_semantics = new Semantics(model, _symbolTable, pi);
		_aggregationSemantics = _semantics.getAggregationSemantics();
		_attributeGroupSemantics = _semantics.getAttributeGroupSemantics();
		_attributeSemantics = _semantics.getAttributeSemantics();
		_expressionSemantics = _semantics.getExpressionSemantics();
		_miscSemantics = _semantics.getMiscSemantics();
		_realAttributeSemantics = _semantics.getRealAttributeSemantics();
		_scopeSemantics = _semantics.getScopeSemantics();
		_tempAttributeSemantics = _semantics.getTempAttributeSemantics();

		_symbolTable.setCheck(_semantics.getCheck());
	}
}

skript
	{
		Token token;
	}
	:
		( token=spaltenDefinition		{_miscSemantics.check(token,_definition, 0);}
		| token=filterDefinition		{_miscSemantics.check(token, _definition, 1);}
		| token=standardsListe		{_miscSemantics.check(token, _definition, 2);}
		| token=definitionenListe 	{_miscSemantics.check(token, _definition, 3);}
		)*
		EOF!
;

spaltenDefinition returns [Token token]
	:	a:Spalten^ Doppelpunkt! 			{token = a;}
			spaltenMitBlock

;

spaltenMitBlock
	:	(mitKonstruktSpalte
		| attributSpalte
		| tempAttributSpalte
		| attributgruppenSpalte )*
;

mitKonstruktSpalte
	:	Mit^ qualifizierung
			spaltenMitBlock
		Ende! Mit!
;

qualifizierung
	//  Objekttyp     :     Attributgruppe  :   Aspekt
	:	Name^ ( Doppelpunkt! Name ( Doppelpunkt! aspektName )? )?
;

aspektName
	:	(Name|NameMitKlammern)
	|	Fragezeichen^ GanzZahl	// Ungebundener Aspekt
;

attributSpalte
{
	String alsName = null;
}
	:	Attribut^ (nichtAuffuellen)? attribut ( alsDefinition )?
	|	Attribut! Zeitdauer^ ( alsName=alsDefinition )?					{_symbolTable.addDurationAttribute(alsName);}
;

attribut
	:	aspektName (Doppelpunkt (aspektName))*
;

alsDefinition returns [String alsName = null]
	:	Als^ z:Zeichenkette							{alsName = z.getText().substring(1,z.getText().length()-1);}
;

tempAttributSpalte
	{
		String alsName = null;
	}
	:	TempAttribut^ n:Name ( alsName=alsDefinition )? 					{_symbolTable.addTempAttribute(n.getText(), alsName);}
;

attributgruppenSpalte
	:	Attributgruppe^ (nichtAuffuellen)? ( Name Doppelpunkt! )* aspektName ( alsDefinition )?
;

filterDefinition returns [Token token]
	:	a:Filter^ Doppelpunkt! 	{token = a;}
			filterMitBlock
;

filterMitBlock
	{
		Token token;
	}
	:	( mitKonstruktFilter
		| aggregationsBlock
	 	| token=einschraenkungsAnweisung	{_miscSemantics.check(token, _filter,0);}
		| token=anwendungsAnweisung			{_miscSemantics.check(token, _filter,1);}
		| token=nachfilterAnweisung 		{_miscSemantics.check(token, _filter,2);}
		)*
;

mitKonstruktFilter
	:	Mit^ qualifizierung
			filterMitBlock
		Ende! Mit!
;

aggregationsBlock
	:	(nichtAuffuellen)? aspektName (Doppelpunkt! aspektName )* Doppelpunkt Aggregation^ ( Name ) ( Komma ( Name ))*
	|	Zeitdauer^ Doppelpunkt! Aggregation! ( Name ) ( Komma ( Name ) )*
;

einschraenkungsAnweisung  returns [Token token=null]
	:	(a1:Einschraenkung^ {token = a1;}| a2:EinschraenkungUml^ {token = a2;})
			( Verschmelzen )? ausdruck
;

anwendungsAnweisung  returns [Token token]
	: a:Anwendung^ anwendungsArt ( Komma! anwendungsArt )* 		{token = a;}
;

anwendungsArt
	:	Gesamt^ ( KlammerAuf! ( Objekt | Spalte ) KlammerZu! )?
		| Intervall^ ( KlammerAuf! ( Objekt | Spalte ) KlammerZu! )?
		| Liste^
;

nachfilterAnweisung returns [Token token]
	:	a:Nachfilter^ Zeichenkette 	{token = a;}
;

standardsListe returns [Token token]
	:	a:Standards^ Doppelpunkt! 	{token = a;}
			standardsMitBlock
;

standardsMitBlock
	:	( mitKonstruktObjekte
		| objektDefinition
		| pseudoObjektDefinition
		| protokollzeitraumDefinition
		| protokollartDefinition
		| standardbindungDefinition
		| unveraendertKennungDefinition)*
;

mitKonstruktObjekte
	:	Mit^ qualifizierung
			standardsMitBlock
			Ende! Mit!
;

objektDefinition
	{
		ArrayList list = new ArrayList();
	}
	:	n:Name^ {list.add(n.getText());} ( Doppelpunkt! m:Name {list.add(m.getText());} )*						{_miscSemantics.addStandardObjects(n,list);}
;

protokollzeitraumDefinition
	: Protokollzeitraum^ ds:Datum Komma! zs:Zeit Minus! de:Datum Komma! ze:Zeit						{_symbolTable.addStandardPeriod(ds.getText(),zs.getText(),de.getText(),ze.getText());}
;

protokollartDefinition
	: Protokollart^( Aenderungsprotokoll | AenderungsprotokollUml | Zustandsprotokoll | Ereignisprotokoll )
;

standardbindungDefinition
	: f:Fragezeichen^ z:GanzZahl Gleich! n:Name				{_symbolTable.addStandardBinding(f.getText() + z.getText(), n.getText());}
;

unveraendertKennungDefinition
	: UnveraendertKennungA^ Pro ( Zeile | Zelle)
	| UnveraendertKennungB^ Pro ( Zeile | Zelle)
;

pseudoObjektDefinition
	:	{ArrayList types = new ArrayList();
		 ArrayList objects = new ArrayList();}
		t:Pseudoobjekt^ KlammerAuf! TY1:Name			{ types.add(TY1.getText()); }
								( Komma! TY2:Name	{ types.add(TY2.getText()); }
								)*
					  KlammerZu!
			Gleich ( KlammerAuf! (O1:Name)?			{ objects.add((O1 == null) ? "" : O1.getText()); O1 = null; }
								( Komma! (O2:Name)? 	{ objects.add((O2 == null) ? "" : O2.getText()); O2 = null;}
								)*
					 KlammerZu!
					)+
													{ _symbolTable.addPseudoObjectDefinition(t, types, objects ); }
;

definitionenListe returns [Token token]
	:	a:Definitionen^ Doppelpunkt! 	{token = a;}
			definitionenMitBlock
;

definitionenMitBlock
	:	( mitKonstruktDefinitionen
		| tempAttributDefinition
		| aggregatsfunktionDefinition
		| aliasDefinition )*
;

mitKonstruktDefinitionen
	:	Mit^ qualifizierung
			definitionenMitBlock
		Ende! Mit!
;

tempAttributDefinition
	:	TempAttribut^ O:Name Doppelpunkt! N:Name Gleich! ausdruck
;

aggregatsfunktionDefinition
	:	a:Aggregationsfunktion^ n:Name Gleich! z:Zeichenkette 		{_symbolTable.addAggregationDefinition(a, n.getText(), z.getText().substring(1,z.getText().length()-1));}
;

aliasDefinition
	:	Alias^ N:Name Gleich! M:Name {_symbolTable.addAliasDefinition(M, N.getText(),M.getText());}
;

ausdruck
	:	oderAusdruck
;

oderAusdruck
	:	undAusdruck (Oder^ undAusdruck)*
;

undAusdruck
	:	gleichheitsAusdruck (Und^ gleichheitsAusdruck)*
;

gleichheitsAusdruck
	:	vergleichsAusdruck ( ( Gleich^ | Ungleich^ ) vergleichsAusdruck)*
;

vergleichsAusdruck
	:	additionsAusdruck ( ( Kleiner^ | Groesser^ | KleinerGleich^ | GroesserGleich^ ) additionsAusdruck )*
;

additionsAusdruck
	:	multiplikationsAusdruck ( ( Plus^ | Minus^ | Ampersand^) multiplikationsAusdruck )*
;

multiplikationsAusdruck
	:	negationsAusdruck ( ( Mal^ | Dividiert^ | Div^ | Modulo^ ) negationsAusdruck )*
;

negationsAusdruck
	:	Nicht^ negationsAusdruck
	|	Minus^ negationsAusdruck
	|	erweiterterAusdruck
;

erweiterterAusdruck
	:	basisAusdruck
	|	erweiterbarerBasisAusdruck
;

basisAusdruck
	:	GanzZahl
	|	KommaZahl
	|	wahrheitswert
	|	Zeichenkette
	|	Zeitdauer
	|	(nichtAuffuellen)? attribut
;

erweiterbarerBasisAusdruck
	:	funktionsAufruf
	|	KlammerAuf! ausdruck KlammerZu!
	|	EckigeKlammerAuf^ ( ausdruck (Komma! ausdruck)* )? EckigeKlammerZu!
;

funktionsAufruf
	:	Potenz^ KlammerAuf! ausdruck Komma! ausdruck KlammerZu!
		| Exp^ KlammerAuf! ausdruck KlammerZu!
		| Ln^ KlammerAuf! ausdruck KlammerZu!
		| Min^ KlammerAuf! ausdruck ( Komma! ausdruck )+ KlammerZu!
		| Max^ KlammerAuf! ausdruck ( Komma! ausdruck )+ KlammerZu!
		| Abs^ KlammerAuf! ausdruck KlammerZu!
		| IstFehler^ KlammerAuf! ausdruck KlammerZu!
		| IstZahl^ KlammerAuf! ausdruck KlammerZu!
		| IstZustand^ KlammerAuf! ausdruck KlammerZu!
		| IstLeer^ KlammerAuf! ausdruck KlammerZu!
		| IstText^ KlammerAuf! ausdruck KlammerZu!
		| IstWahrheitswert^ KlammerAuf! ausdruck KlammerZu!
;


wahrheitswert
	:	Falsch | Wahr
;

nichtAuffuellen
	: NichtAuffuellenA | NichtAuffuellenB
;

unveraendertKennung
	: (UnveraendertKennungA | UnveraendertKennungB)
;

/**
 * @author Kappich + Kniss Systemberatung Verkehr und Technik
 * @author O. Weiss
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $$Revision: 1.9 $$ / $$Date: 2007/12/17 15:22:59 $$ / ($$Author: martinh $$)
 */
class ProtocolLexer extends Lexer;
options {
	k = 6;
	importVocab=ProtocolTokens;
	charVocabulary = '\3'..'\377' | '\u00fc' | '\u00dc';
}

tokens {
	Aenderungsprotokoll			= "aenderungsprotokoll";
	AenderungsprotokollUml		= "\u00e4nderungsprotokoll";
	Abs							= "abs";
	Aggregation					= "aggregation";
	Aggregationsfunktion		= "aggregationsfunktion";
	Alias						= "alias";
	Als							= "als";
	Anwendung					= "anwendung";
	Attribut					= "attribut";
	Attributgruppe				= "attributgruppe";
	TempAttribut				= "tempAttribut";
	Definitionen				= "definitionen";
	Div							= "div";
	Einschraenkung				= "einschraenkung";
	EinschraenkungUml			= "einschr\u00e4nkung";
	Ende						= "ende";
	Ereignisprotokoll			= "ereignisprotokoll";
	Exp							= "exp";
	Falsch						= "falsch";
	Filter						= "filter";
	Gesamt						= "gesamt";
	Intervall					= "intervall";
	IstFehler					= "istFehler";
	IstLeer 					= "istLeer";
	IstText	    				= "istText";
	IstWahrheitswert			= "istWahrheitswert";
	IstZahl					    = "istZahl";
	IstZustand					= "istZustand";
	Liste						= "liste";
	Ln							= "ln";
	Max							= "max";
	Min							= "min";
	Mit							= "mit";
	Modulo						= "modulo";
	Nachfilter					= "nachfilter";
	Nicht						= "nicht";
	NichtAuffuellenA			= "nichtauffuellen";
	NichtAuffuellenB 			= "nichtauff\u00fcllen";
	Objekt						= "objekt";
	Oder						= "oder";
	Potenz						= "potenz";
	Protokollart				= "protokollart";
	Protokollzeitraum			= "protokollzeitraum";
	Pseudoobjekt				= "pseudoobjekt";
	Pro		    		        = "pro";
	Zeile		    	    	= "zeile";
	Zelle		    	    	= "zelle";
	Spalten						= "spalten";
	Standards					= "standards";
	Spalte						= "spalte";
	Und							= "und";
	UnveraendertKennungA		= "unveraendertkennung";
	UnveraendertKennungB 		= "unver\u00e4ndertkennung";
	Verschmelzen				= "verschmelzen";
	Wahr						= "wahr";
	Zeitdauer					= "zeitdauer";
	Zustandsprotokoll			= "zustandsprotokoll";
}

Ampersand				:	'&' ;
Dividiert				:	'/' ;
Doppelpunkt				:	':'	;
EckigeKlammerAuf		:	'['	;
EckigeKlammerZu			:	']'	;
Fragezeichen			:	'?' ;
Gleich					:	'=' ;
Groesser				:	'>' ;
GroesserGleich			:	Groesser Gleich ;
KlammerAuf				:	'(' ;
KlammerZu				:	')' ;
Kleiner					:	'<' ;
KleinerGleich			:	Kleiner Gleich ;
Komma					:	',' ;
Mal						:	'*' ;
Minus					:	'-' ;
Plus					:	'+' ;
Prozent					:	'%' ;
Punkt					:	'.'	;
Ungleich				:	Kleiner Groesser;


NameOderNameMitKlammern
	: (( 'a'..'z' | 'A'..'Z' | '_' | Umlaut ) ( 'a'..'z' | 'A'..'Z' | '_' | '.' | '-' | Umlaut | '0'..'9')* ('[' | ']') ( 'a'..'z' | 'A'..'Z' | '_' | '.' | '-' | Umlaut | '0'..'9'| '[' | ']')*) => NameMitKlammern {$setType(NameMitKlammern);}
	| (( 'a'..'z' | 'A'..'Z' | '_' | Umlaut ) ( 'a'..'z' | 'A'..'Z' | '_' | '.' | '-' | Umlaut | '0'..'9')*) => Name {$setType(Name);}
;

protected Name
	:	( 'a'..'z' | 'A'..'Z' | '_' | Umlaut ) ( 'a'..'z' | 'A'..'Z' | '_' | '.' | '-' | Umlaut | '0'..'9')*
;

protected NameMitKlammern
	:	( 'a'..'z' | 'A'..'Z' | '_' | Umlaut ) ( 'a'..'z' | 'A'..'Z' | '_' | '.' | '-' | Umlaut | '0'..'9'| '[' | ']')*
;

SL_COMMENT
	:	"#" ( ~( '\n' | '\r' ) )*
	{ $setType(Token.SKIP); }
;

DatumOderZahl		//we need this syntactic predicate to distinguish between dates & numbers...
	: ( Ziffer Ziffer '.' Ziffer Ziffer '.' Ziffer Ziffer Ziffer Ziffer ) => Datum {$setType(Datum);}
	| (Ziffer Ziffer ':' Ziffer Ziffer ':' Ziffer Ziffer ',' Ziffer Ziffer Ziffer) => Zeit {$setType(Zeit);}
	| ( ( Ziffer )+ '.' ) => KommaZahl {$setType(KommaZahl);}
	| ( (Ziffer)* ( 'e' | 'E' ) ) => KommaZahl {$setType(KommaZahl);}
	| GanzZahl {$setType(GanzZahl);}
;

protected Nicht
	: "nicht"
;


protected Zeit
	:	Ziffer Ziffer ':' Ziffer Ziffer ':' Ziffer Ziffer ',' Ziffer Ziffer Ziffer
;

protected Datum
	:	Ziffer Ziffer '.' Ziffer Ziffer '.' Ziffer Ziffer Ziffer Ziffer
;

protected GanzZahl
	:	( Ziffer )+
;

protected KommaZahl
	:	( Ziffer )+ ( ( '.' ) ( Ziffer )+ )? ( ( 'e' | 'E' ) ( '-' )? ( Ziffer )+ )?
;

protected Ziffer
	:	( '0'..'9' )
;

Zeichenkette
	:	'"' ( Steuerungszeichen | ~( '"' | '\n' | '\r'| '\\' ) )* '"'
;

protected Steuerungszeichen
	:	'\\'
		 (	'n'   { $setText("\n"); }
	        | '"' { $setText("\""); }
	        | '\\' { $setText("\\"); }
	     )
;

WS	:	( ' ' | '\t' | '\\' ( ' ' | '\t' )* (SL_COMMENT)? ZeilenEnde )+
	{ $setType(Token.SKIP); }
;

ZeilenEnde
	:	( '\n' | '\r' | "\r\n" )
	{ newline(); $setType(Token.SKIP); }
;

protected Umlaut
	// ae Ae oe Oe ue Ue sz
	:	'\u00e4' | '\u00c4' | '\u00f6' | '\u00d6' | '\u00fc' | '\u00dc' | '\u00df' ;


/*
Der Baum wird vom TreeWalker nicht verändert, da es kaum Nutzen bringt.
Zwei Läufe sind immer notwendig, da z.B im Definitionen-Bereich temporäre Attribute definiert werden; diese
greifen auf Reale attribute zurück, dafür muss es die Bindungen aus dem Standards Bereich schon geben!
=> Man muss so oder so bereichsübergreifend arbeiten (d.h. schon bestimmte Informationen beim Parsen in der Symboltabelle ablegen)
*/
/**
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $$Revision: 1.9 $$ / $$Date: 2007/12/17 15:22:59 $$ / ($$Author: martinh $$)
 */
class ProtocolTreeWalker extends TreeParser;
options
{
	exportVocab=ProtocolTokens;
	k=1;
		defaultErrorHandler=false;	//Andernfalls werden vom Parser keine Exceptions geworfen, und der Skript-Interpreter merkt nicht, dass es zu einem Fehler kam.
}
{
	//Zum überprüfen der Semantik
	private Semantics _semantics;
	private AggregationSemantics _aggregationSemantics;
	private AttributeGroupSemantics _attributeGroupSemantics;
	private AttributeSemantics _attributeSemantics;
	private ExpressionSemantics _expressionSemantics;
	private MiscellaneousSemantics _miscSemantics;
	private RealAttributeSemantics _realAttributeSemantics;
	private ScopeSemantics _scopeSemantics;
	private TempAttributeSemantics _tempAttributeSemantics;

	//Freie Bindungen sollen gebunden werden
	private boolean _noFreeAspects = false;

	public ProtocolTreeWalker(Semantics semantics)
	{
		super();
		this._semantics = semantics;
		_aggregationSemantics = _semantics.getAggregationSemantics();
		_attributeGroupSemantics = _semantics.getAttributeGroupSemantics();
		_attributeSemantics = _semantics.getAttributeSemantics();
		_expressionSemantics = _semantics.getExpressionSemantics();
		_miscSemantics = _semantics.getMiscSemantics();
		_realAttributeSemantics = _semantics.getRealAttributeSemantics();
		_scopeSemantics = _semantics.getScopeSemantics();
		_tempAttributeSemantics = _semantics.getTempAttributeSemantics();
	}
}

skript
	:	( spaltenDefinition
		| filterDefinition
		| standardsListe
		| definitionenListe
		)*
;

spaltenDefinition
	:	#(Spalten spaltenMitBlock)
;

spaltenMitBlock
	:	(mitKonstruktSpalte
		| attributSpalte
		| tempAttributSpalte
		| attributgruppenSpalte )*
;

mitKonstruktSpalte
	{
		List<String> path = null;
	}
	:	#(Mit	path=P:qualifizierung					{ _scopeSemantics.push(P,path);}
				spaltenMitBlock 						{ _scopeSemantics.pop(); }
				)
;

qualifizierung returns [List<String> path = new ArrayList<String>()]
	:	#(o:Name {path.add(o.getText());} ( a:Name {path.add(a.getText());} ( aspektName[path] )? )? )
;

attributSpalte
	{
		ArrayList<String> v = new ArrayList<String>();
		String alsName = null, y = null;
	}
	:	#(Attribut (f1:nichtAuffuellen)? ast:attributAngabe[v] (alsName=alsDefinition)?) {_realAttributeSemantics.addRealAttribute(ast, v, alsName, (f1==null));}
	|   #(Zeitdauer (alsDefinition)?)	                                                 {_miscSemantics.addDurationAttribute();}
;


alsDefinition returns [String s = null]
	: #(Als n:Zeichenkette)
	{
		s = n.getText().substring(1,n.getText().length()-1);
	}
;

aspektName[List v]
{if (v == null) v = new ArrayList();}
	: (n:Name|n2:NameMitKlammern)							{v.add(n != null? n.getText() : n2.getText());}
	| #( f:Fragezeichen z:GanzZahl)			{
											String aspect = _miscSemantics.getBinding(f.getText()+z.getText());
											v.add(aspect);
										}
;

tempAttributSpalte
	{
		String alsName = null;
	}
	:	#( TempAttribut n:Name ( alsName=alsDefinition )? 		 )		{_tempAttributeSemantics.addTempAttributePlaceholder(n.getText(), alsName);}
;

attributgruppenSpalte
	{
		ArrayList<String> v = new ArrayList<String>();
		String alsName = null;
	}
		//nicht ganz korrekt, müsste eigentlich "Name* aspektName" sein, aber da achtet der Parser schon darauf...
	:	#(a:Attributgruppe (na:nichtAuffuellen)?( aspektName[v] )+ ( alsName=alsDefinition )? )	{_attributeGroupSemantics.addAttributeGroup(a, v, alsName, (na==null));}
;


filterDefinition
	:	#(Filter filterMitBlock)
;

filterMitBlock
	:	(mitKonstruktFilter
		| aggregationsBlock
		| einschraenkungsAnweisung
		| anwendungsAnweisung
		| nachFilterAnweisung
		 )*
;

mitKonstruktFilter
	{
		List<String> path = null;
	}
	:	#(Mit	path=P:qualifizierung			{ _scopeSemantics.push(P, path);}
				filterMitBlock 					{ _scopeSemantics.pop(); }
		)
;

aggregationsBlock	//eigentlich nicht ganz korrekt, da das letzte Element ein "name" sein muss. Hier aber egal, da der Parser das schon überprüft hat
	{
		ArrayList<String> attrList = new ArrayList<String>();
		ArrayList<String> aggrList = new ArrayList<String>();
	}	//Aggregation Zeitdauer Attribut
		: #( az:Zeitdauer aspektName[aggrList] (Komma aspektName[aggrList] )* ) 						{_aggregationSemantics.addAggregationForDuration(az, aggrList);}
		//Aggregation "normale" Attribute
		| #( a:Aggregation (na:nichtAuffuellen)? (aspektName[attrList])+ Doppelpunkt aspektName[aggrList] (Komma aspektName[aggrList] )* )						{_aggregationSemantics.addAggregation(a, attrList, aggrList, (na==null));}
;

einschraenkungsAnweisung
	{
		ExpressionTree tree = null;
		boolean merge = false;
	}
	:	#( Einschraenkung (Verschmelzen {merge = true;} )? tree=ausdruck[false] )		{_miscSemantics.setConstraint(tree, merge);}
		| #( EinschraenkungUml (Verschmelzen {merge = true;} )? tree=ausdruck[false] )		{_miscSemantics.setConstraint(tree, merge);}
;


anwendungsAnweisung
	{
		HashSet<ProcessingInformation.ApplyAggregation> aggregationList = new HashSet<ProcessingInformation.ApplyAggregation>();
		ProcessingInformation.ApplyAggregation agg = null;
	}
	:	#(Anwendung (agg=anwendungsArt {aggregationList.add(agg);} )+ )		{_aggregationSemantics.setOrder(aggregationList);}
;

anwendungsArt returns [ProcessingInformation.ApplyAggregation agg = ProcessingInformation.ApplyAggregation.LISTE]
	:	#(Gesamt {agg = ProcessingInformation.ApplyAggregation.GESAMT_SPALTE;}
			 (Objekt {agg = ProcessingInformation.ApplyAggregation.GESAMT_OBJEKT;} | Spalte)? )
		| #(Intervall {agg = ProcessingInformation.ApplyAggregation.INTERVALL_SPALTE;}
			(Objekt {agg = ProcessingInformation.ApplyAggregation.INTERVALL_OBJEKT;}| Spalte)? )
		| #(Liste (.)* {agg = ProcessingInformation.ApplyAggregation.LISTE;})
;

nachFilterAnweisung
	: #(Nachfilter z:Zeichenkette)		{_miscSemantics.setPostFilter(z, z.getText().substring(1, z.getText().length()-1));}	// die '"' müssen weg
;

standardsListe
	:	#(Standards standardsMitBlock)
;

standardsMitBlock
	:	(mitKonstruktObjekte
		| objektDefinition
		| pseudoObjektDefinition
		| protokollzeitraumDefinition
		| protokollartDefinition
		| standardbindungDefinition
		| unveraendertKennungDefinition
		)*
;

mitKonstruktObjekte
	{
		List<String> path = null;
	}
	:	#(Mit	path=P:qualifizierung				{ _scopeSemantics.push(P, path);}
				standardsMitBlock 					{ _scopeSemantics.pop(); }
		)
;

objektDefinition
	:	#(Name (Name)* )
;

protokollzeitraumDefinition
	:	#(Protokollzeitraum Datum Zeit Datum Zeit)
;

protokollartDefinition
	:	#(Protokollart (
						Aenderungsprotokoll					{_miscSemantics.setProtocolType(ProtocolType.DeltaProtocol);}
						| AenderungsprotokollUml			{_miscSemantics.setProtocolType(ProtocolType.DeltaProtocol);}
						| Zustandsprotokoll					{_miscSemantics.setProtocolType(ProtocolType.StatusProtocol);}
						| Ereignisprotokoll                 {_miscSemantics.setProtocolType(ProtocolType.EventProtocol);}
						)
		)
;

unveraendertKennungDefinition
	:   #(UnveraendertKennungA Pro (
								  Zeile         {_miscSemantics.setNoChangeMarker(NoChangeMarker.Row);}
								  | Zelle         {_miscSemantics.setNoChangeMarker(NoChangeMarker.Cell);}
								 )
	)
	|   #(UnveraendertKennungB Pro (
								  Zeile         {_miscSemantics.setNoChangeMarker(NoChangeMarker.Row);}
								  | Zelle         {_miscSemantics.setNoChangeMarker(NoChangeMarker.Cell);}
								 )
	)
;

standardbindungDefinition
	:	#(Fragezeichen GanzZahl Name)
;

pseudoObjektDefinition
	:	#(Pseudoobjekt (.)+ )
;

definitionenListe
	:	#(Definitionen definitionenMitBlock)
;

definitionenMitBlock
	:	( mitKonstruktDefinitionen
		| tempAttributDefinition
		| aggregatsfunktionDefinition
		| aliasDefinition
		)*
;

mitKonstruktDefinitionen
	{
		List<String> path = null;
	}
	:	#(Mit	path=P:qualifizierung					{ _scopeSemantics.push(P, path);}
				definitionenMitBlock 					{ _scopeSemantics.pop(); }
		)
;

tempAttributDefinition
	{
		ExpressionTree tree = null;
	}
	:	#( TempAttribut n:Name t:Name tree=ausdruck[true] )		{_tempAttributeSemantics.addTempAttributeDefinition(n, n.getText(),t.getText(),tree);}
;


aggregatsfunktionDefinition
	:	#(Aggregationsfunktion Name Zeichenkette)
;

aliasDefinition
	:	#(Alias Name Name)
;

ausdruck[boolean b] returns [ExpressionTree tree = new ExpressionTree()]
	{ExpressionTree left, right;}
	:	#(Oder left=ausdruck[b] right=ausdruck[b])				{tree.operation = ExpressionTree.Operation.or;tree.left = left; tree.right = right;}
		| #(Und left=ausdruck[b] right=ausdruck[b])				{tree.operation = ExpressionTree.Operation.and;tree.left = left; tree.right = right;}
		| #(Gleich left=ausdruck[b] right=ausdruck[b])			{tree.operation = ExpressionTree.Operation.equal;tree.left = left; tree.right = right;}
		| #(Ungleich left=ausdruck[b] right=ausdruck[b])			{tree.operation = ExpressionTree.Operation.notEqual;tree.left = left; tree.right = right;}
		| #(Kleiner left=ausdruck[b] right=ausdruck[b])			{tree.operation = ExpressionTree.Operation.less;tree.left = left; tree.right = right;}
		| #(Groesser left=ausdruck[b] right=ausdruck[b])			{tree.operation = ExpressionTree.Operation.greater;tree.left = left; tree.right = right;}
		| #(KleinerGleich left=ausdruck[b] right=ausdruck[b])		{tree.operation = ExpressionTree.Operation.lessEqual;tree.left = left; tree.right = right;}
		| #(GroesserGleich left=ausdruck[b] right=ausdruck[b])	{tree.operation = ExpressionTree.Operation.greaterEqual;tree.left = left; tree.right = right;}
		| #(Plus left=ausdruck[b] right=ausdruck[b])				{tree.operation = ExpressionTree.Operation.add;tree.left = left; tree.right = right;}
		| ( #(Minus ausdruck[b] ausdruck[b]) ) => #(Minus left=ausdruck[b] right=ausdruck[b])				{tree.operation = ExpressionTree.Operation.subtract;tree.left = left; tree.right = right;}
		| #(Ampersand left=ausdruck[b] right=ausdruck[b])			{tree.operation = ExpressionTree.Operation.conc;tree.left = left; tree.right = right;}
		| #(Mal left=ausdruck[b] right=ausdruck[b])				{tree.operation = ExpressionTree.Operation.mult;tree.left = left; tree.right = right;}
		| #(Dividiert left=ausdruck[b] right=ausdruck[b])			{tree.operation = ExpressionTree.Operation.div;tree.left = left; tree.right = right;}
		| #(Div left=ausdruck[b] right=ausdruck[b])				{tree.operation = ExpressionTree.Operation.divInt;tree.left = left; tree.right = right;}
		| #(Modulo left=ausdruck[b] right=ausdruck[b])			{tree.operation = ExpressionTree.Operation.modulo;tree.left = left; tree.right = right;}
			//mit ganz und Kommanzahlen muss gesondert umgegangen werden: Beim Erzeugen muss klar sein, ob sie positiv oder negativ - anderenfalls kann es Probleme beim Erzeugen des JavaObjektes geben (abs(kleinster long)) > (größter long) (-> wäre beim erzeugen zu groß!)
		| (#(Minus GanzZahl))=>#(Minus n:GanzZahl)					{tree = ExpressionLeaf.createLongLeaf(n, "-" + n.getText());}
		| (#(Minus KommaZahl))=>#(Minus m:KommaZahl)					{tree = ExpressionLeaf.createDoubleLeaf(m, "-" + m.getText());}
		| #(Minus left = ausdruck[b])							{tree.operation = ExpressionTree.Operation.neg;tree.left = left;}
		| #(Nicht left=ausdruck[b])							{tree.operation = ExpressionTree.Operation.not;tree.left = left;}
		| left=erweiterterAusdruck[b]							{tree = left;}
;

erweiterterAusdruck[boolean b] returns [ExpressionLeaf leaf = null]
	:	leaf=basisAusdruck[b]
	|	leaf=erweiterbarerBasisAusdruck[b]
;

basisAusdruck[boolean b] returns [ExpressionLeaf leaf = null]
	:	z:GanzZahl						{leaf = ExpressionLeaf.createLongLeaf(z, z.getText());}
	|	x:KommaZahl						{leaf = ExpressionLeaf.createDoubleLeaf(x, x.getText());}
	|	leaf=wahrheitswert
	|	s:Zeichenkette				{leaf = new ExpressionLeaf(s.getText().substring(1,s.getText().length()-1));}
	|	leaf=attribut[b]
	|	Zeitdauer					{leaf = new ExpressionLeaf(DurationAttributeDescription.forHash());}
;

attribut[boolean b] returns [ExpressionLeaf leaf = null]
	{
		ArrayList<String> v = new ArrayList<String>();
	}
	: 	 (nichtAuffuellen:nichtAuffuellen)? ast:attributAngabe[v] 		{leaf = new ExpressionLeaf(_expressionSemantics.createExpressionAttribute(ast, v, b, nichtAuffuellen));}
;

attributAngabe[List v]
	:	aspektName[v] (Doppelpunkt (aspektName[v]))*
;

erweiterbarerBasisAusdruck[boolean b] returns [ExpressionLeaf leaf = null]
	:	leaf=funktionsAufruf[b]
	|	#(EckigeKlammerAuf ( ausdruck[b] (ausdruck[b])* )? )
;

funktionsAufruf[boolean b] returns [ExpressionLeaf leaf = null]
	{
		ArrayList<ExpressionTree> list = new ArrayList<ExpressionTree>();
		ExpressionTree x = null;
		ExpressionTree y = null;
	}
	:	#(Potenz x=ausdruck[b] y=ausdruck[b])											{list.add(x); list.add(y); leaf = new ExpressionLeaf("de.bsvrz.pua.prot.functions.Power", list);}
		| #(Exp  x=ausdruck[b] )													{list.add(x); leaf = new ExpressionLeaf("de.bsvrz.pua.prot.functions.Exp", list);}
		| #(Ln 	 x=ausdruck[b] )													{list.add(x); leaf = new ExpressionLeaf("de.bsvrz.pua.prot.functions.Ln", list);}
		| #(Min  x=ausdruck[b] {list.add(x);} ( y=ausdruck[b] {list.add(y);} )+ )		{leaf = new ExpressionLeaf("de.bsvrz.pua.prot.functions.Min", list);}
		| #(Max  x=ausdruck[b] {list.add(x);} ( y=ausdruck[b] {list.add(y);} )+ )		{leaf = new ExpressionLeaf("de.bsvrz.pua.prot.functions.Max", list);}
		| #(Abs  x=ausdruck[b] )													{list.add(x); leaf = new ExpressionLeaf("de.bsvrz.pua.prot.functions.Abs", list);}
		| #(IstFehler  x=ausdruck[b] )													{list.add(x); leaf = new ExpressionLeaf("de.bsvrz.pua.prot.functions.IsError", list);}
		| #(IstLeer  x=ausdruck[b] )													{list.add(x); leaf = new ExpressionLeaf("de.bsvrz.pua.prot.functions.IsEmpty", list);}
		| #(IstText  x=ausdruck[b] )													{list.add(x); leaf = new ExpressionLeaf("de.bsvrz.pua.prot.functions.IsText", list);}
		| #(IstWahrheitswert  x=ausdruck[b] )													{list.add(x); leaf = new ExpressionLeaf("de.bsvrz.pua.prot.functions.IsBoolean", list);}
		| #(IstZahl  x=ausdruck[b] )													{list.add(x); leaf = new ExpressionLeaf("de.bsvrz.pua.prot.functions.IsNumber", list);}
		| #(IstZustand  x=ausdruck[b] )													{list.add(x); leaf = new ExpressionLeaf("de.bsvrz.pua.prot.functions.IsState", list);}
;


wahrheitswert returns [ExpressionLeaf leaf = null]
	:	Falsch		{leaf = new ExpressionLeaf(false);}
		| Wahr		{leaf = new ExpressionLeaf(true);}
;

nichtAuffuellen
	: NichtAuffuellenA | NichtAuffuellenB
;
